@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the Linux console (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2003 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -34,36 +34,35 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <stdio.h>
-#include <sys/termios.h>
 #include <string.h>
 
-#define BL_NEED_ARRAYS
-#include "Programs/brl.h"
 #include "Programs/misc.h"
 #include "Programs/message.h"
 
 typedef enum {
-  PARM_BAUDRATE=0,
+  PARM_BAUDRATE,
   PARM_KBEMU
 } DriverParameter;
 #define BRLPARMS "baudrate", "kbemu"
 
 #include "Programs/brl_driver.h"
+#include "Programs/tbl.h"
 #include "braille.h"
-#include "bindings.h"		/* for keybindings */
+#include "Programs/serial.h"
 
+#define BL_NEED_ARRAYS
+#include "bindings.h"		/* for keybindings */
 
 #define QSZ 256			/* size of internal input queue in bytes */
 #define INT_CSR_SPEED 2		/* on/off time in cycles */
 #define ACK_TIMEOUT 1000	/* timeout in ms for an ACK to come back */
 
-int blite_fd = -1;		/* file descriptor for Braille display */
+SerialDevice *BL_serialDevice = NULL;		/* file descriptor for Braille display */
 
 static TranslationTable outputTable;	/* dot mapping table (output) */
 static TranslationTable inputTable;	/* mapping for reversed display */
 static unsigned char *prevdata = NULL;	/* previously received data */
 static unsigned char *rawdata = NULL;	/* writebrl() buffer for raw Braille data */
-static struct termios oldtio;	/* old terminal settings */
 static int blitesz;	/* set to 18 or 40 */
 static int waiting_ack = 0;	/* waiting acknowledgement flag */
 static int reverse_kbd = 0;	/* reverse keyboard flag */
@@ -194,7 +193,7 @@ qget (blkey * kp)
 	{
 	  kp->raw = c3;
 	  if (c3 & 0x80)
-            kp->cmd = barcmds[c3 & 0xF];
+            kp->cmd = (*barcmds)[c3 & 0xF];
 	  else if (c3 > 0 && c3 <= blitesz)
 	    kp->routing = c3;
 	}
@@ -219,7 +218,7 @@ qget (blkey * kp)
       kp->raw = c3;
 
       if (c3 & 0x0f)
-        kp->cmd = barcmds[((c3 & 0x1) << 3) | ((c3 & 0x2) << 1) | ((c3 & 0x4) >> 1) | ((c3 & 0x8) >> 3)];
+        kp->cmd = (*barcmds)[((c3 & 0x1) << 3) | ((c3 & 0x2) << 1) | ((c3 & 0x4) >> 1) | ((c3 & 0x8) >> 3)];
       else if (c3 & 0x30)
         kp->cmd = rwwcmds[(c3 >> 4) & 0x3];
       else if (c3 & 0xc0)
@@ -247,7 +246,7 @@ qfill (void)
 {
   unsigned char c;		/* character buffer */
 
-  while (read (blite_fd, &c, 1))
+  while (serialReadData (BL_serialDevice, &c, 1, 0, 0) == 1)
     {
       if (waiting_ack && c == 5)	/* ^e is the acknowledgement character ... */
 	waiting_ack = 0;
@@ -277,32 +276,32 @@ brl_identify (void)
 
 static int
 await_ack (void) {
-  timeout_yet(0);
+  hasTimedOut(0);
   waiting_ack = 1;
   do {
-    delay(10);	/* sleep for 10 ms */
+    approximateDelay(10);	/* sleep for 10 ms */
     qfill();
     if (!waiting_ack) return 1;
-  } while (!timeout_yet(ACK_TIMEOUT));
+  } while (!hasTimedOut(ACK_TIMEOUT));
   return 0;
 }
 
 static void
 write_prebrl (void) {
   static const unsigned char request[] = {0X05, 0X44};			/* code to send before Braille */
-  write(blite_fd, request, sizeof(request));
+  serialWriteData(BL_serialDevice, request, sizeof(request));
 }
 
 static int
-brl_open (BrailleDisplay *brl, char **parameters, const char *brldev)
+brl_open (BrailleDisplay *brl, char **parameters, const char *device)
 {
-  static const unsigned good_baudrates[] =
+  static const int good_baudrates[] =
     {300,600,1200,2400,4800,9600,19200,38400, 0};
-  speed_t baudrate;
+  int baudrate;
   /* Init string for Model detection */
 
   if (!*parameters[PARM_BAUDRATE] ||
-      !validateBaud(&baudrate, "baud rate",
+      !serialValidateBaud(&baudrate, "baud rate",
 		    parameters[PARM_BAUDRATE], good_baudrates))
     baudrate = BAUDRATE;
 
@@ -317,46 +316,30 @@ brl_open (BrailleDisplay *brl, char **parameters, const char *brldev)
     reverseTranslationTable(&outputTable, &inputTable);
   }
 
+  if (!isSerialDevice(&device)) {
+    unsupportedDevice(device);
+    return 0;
+  }
+
   if ((qbase = (unsigned char *) malloc(QSZ))) {
     /* Open the Braille display device for random access */
-    LogPrint(LOG_DEBUG, "Opening serial port: %s", brldev);
-    if (openSerialDevice(brldev, &blite_fd, &oldtio)) {
-      struct termios newtio;	/* new terminal settings */
-      newtio.c_cflag = CRTSCTS | CS8 | CLOCAL | CREAD;
-      newtio.c_iflag = IGNPAR;
-      newtio.c_oflag = 0;		/* raw output */
-      newtio.c_lflag = 0;		/* don't echo or generate signals */
-      newtio.c_cc[VMIN] = 0;	/* set nonblocking read */
-      newtio.c_cc[VTIME] = 0;
-
+    LogPrint(LOG_DEBUG, "Opening serial port: %s", device);
+    if ((BL_serialDevice = serialOpenDevice(device))) {
       /* activate new settings */
-      if (resetSerialDevice(blite_fd, &newtio, baudrate)) {
-#ifdef DETECT_FOREVER
-        /* wait forever method */
-        while (1) {
-          qflush();
-          write_prebrl();
-          waiting_ack = 1;
-          delay(100);
-          qfill();
-          if (!waiting_ack) break;
-          delay(2000);
-        }
-        if (1) {
-#else /* DETECT_FOREVER */
+      if (serialRestartDevice(BL_serialDevice, baudrate)) {
         qflush();
         write_prebrl();
         if (await_ack()) {
-#endif /* DETECT_FOREVER */
           LogPrint(LOG_DEBUG, "Got response.");
 
-          /* Next, let's detect the BLT-Model (18 || 40). */
+          /* Next, let's detect the BLT-Model (18, 40, M20, M40). */
+          barcmds = &bar2cmds;
           {
             unsigned char cells[18];
             memset(cells, 0, sizeof(cells));
-            write(blite_fd, cells, sizeof(cells));
+            serialWriteData(BL_serialDevice, cells, sizeof(cells));
             waiting_ack = 1;
-            delay(400);
+            approximateDelay(400);
             qfill();
             if (waiting_ack) {
               /* no response, so it must be BLT40 */
@@ -370,11 +353,11 @@ brl_open (BrailleDisplay *brl, char **parameters, const char *brldev)
 
           {
             static const unsigned char request[] = {0X05, 0X57};			/* code to send before Braille */
-            delay(200);
+            approximateDelay(200);
             qflush();
-            write(blite_fd, request, sizeof(request));
+            serialWriteData(BL_serialDevice, request, sizeof(request));
             waiting_ack = 0;
-            delay(200);
+            approximateDelay(200);
             qfill();
             if (qlen) {
               unsigned char response[qlen + 1];
@@ -387,6 +370,13 @@ brl_open (BrailleDisplay *brl, char **parameters, const char *brldev)
               } while (qlen);
               response[length] = 0;
               LogPrint(LOG_INFO, "Braille Lite identity: %s", response);
+
+              if ((response[0] == 'X') &&
+                  (response[1] == ' ') &&
+                  (response[2] == 'B')) {
+                blitesz = atoi(&response[3]);
+                if (blitesz <= 20) barcmds = &bar1cmds;
+              }
             }
           }
 
@@ -395,14 +385,14 @@ brl_open (BrailleDisplay *brl, char **parameters, const char *brldev)
           brl->y = 1;		/* Braille Lites are single line displays */
 
           /* Allocate space for buffers */
-          if ((prevdata = (unsigned char *) malloc(brl->x))) {
+          if ((prevdata = malloc(brl->x))) {
             memset(prevdata, 0, brl->x);
-            if ((rawdata = (unsigned char *) malloc(brl->x))) {
-              return 1;
-            /*
+
+            if ((rawdata = malloc(brl->x))) {
+              if (serialSetFlowControl(BL_serialDevice, SERIAL_FLOW_HARDWARE)) return 1;
+
               free(rawdata);
               rawdata = NULL;
-            */
             } else {
               LogPrint(LOG_ERR, "Cannot allocate rawdata.");
             }
@@ -412,12 +402,11 @@ brl_open (BrailleDisplay *brl, char **parameters, const char *brldev)
             LogPrint(LOG_ERR, "Cannot allocate prevdata.");
           }
         } else {
-          LogPrint(LOG_WARNING, "BLT doesn't seem to be connected, giving up!");
+          LogPrint(LOG_DEBUG, "BrailleLite not responding.");
         }
-        tcsetattr(blite_fd, TCSANOW, &oldtio);
       }
-      close(blite_fd);
-      blite_fd = -1;
+      serialCloseDevice(BL_serialDevice);
+      BL_serialDevice = NULL;
     }
     free(qbase);
     qbase = NULL;
@@ -446,10 +435,9 @@ brl_close (BrailleDisplay * brl)
     qbase = NULL;
   }
 
-  if (blite_fd != -1) {
-    tcsetattr(blite_fd, TCSADRAIN, &oldtio);	/* restore terminal settings */
-    close(blite_fd);
-    blite_fd = -1;
+  if (BL_serialDevice) {
+    serialCloseDevice(BL_serialDevice);
+    BL_serialDevice = NULL;
   }
 }
 
@@ -478,8 +466,8 @@ brl_writeWindow (BrailleDisplay * brl)
       static int timer = 0;		/* for internal cursor */
       timer = (timer + 1) % (INT_CSR_SPEED * 2);
       brl->buffer[int_cursor - 1] = (timer < INT_CSR_SPEED)?
-                                      (B1 | B2 | B3 | B7):
-                                      (B4 | B5 | B6 | B8);
+                                      (BRL_DOT1 | BRL_DOT2 | BRL_DOT3 | BRL_DOT7):
+                                      (BRL_DOT4 | BRL_DOT5 | BRL_DOT6 | BRL_DOT8);
     }
 
   /* Next we must handle display reversal: */
@@ -512,14 +500,14 @@ brl_writeWindow (BrailleDisplay * brl)
       if (!await_ack()) return;
 
       /* OK, now we'll suppose we're all clear to send Braille data. */
-      write(blite_fd, rawdata, blitesz);
+      serialWriteData(BL_serialDevice, rawdata, blitesz);
       await_ack();
     }
 }
 
 
 static int
-brl_readCommand (BrailleDisplay *brl, DriverCommandContext cmds)
+brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context)
 {
   static enum {
     ST_NORMAL,	/* transparent */
@@ -537,7 +525,7 @@ brl_readCommand (BrailleDisplay *brl, DriverCommandContext cmds)
 #endif /* USE_TEXTTRANS */
   static blkey key;
   static unsigned char outmsg[41];
-  int temp = CMD_NOOP;
+  int temp = BRL_CMD_NOOP;
 
  again:
   if(repeatNext || repeat == 0) {
@@ -568,10 +556,10 @@ brl_readCommand (BrailleDisplay *brl, DriverCommandContext cmds)
 	  /* I thought I was smart when I suggested to remove CMD_CUT_END.
 	     Well now there's this nasty exception: the command offset
 	     depends on the display size! */
-	  if(key.cmd == CR_CUTRECT || key.cmd == CR_CUTLINE)
+	  if(key.cmd == BRL_BLK_CUTRECT || key.cmd == BRL_BLK_CUTLINE)
 	    key.cmd += blitesz-1;
 
-	  if(key.spcbar && (key.cmd &VAL_BLK_MASK) == VAL_PASSKEY) {
+	  if(key.spcbar && (key.cmd &BRL_MSK_BLK) == BRL_BLK_PASSKEY) {
           /*
 	    if(!kbemu)
 	      return EOF;
@@ -589,7 +577,7 @@ brl_readCommand (BrailleDisplay *brl, DriverCommandContext cmds)
 	    return key.cmd;
 
 	  /* kbemu could be on, then go on */
-	  if (kbemu && cmds == CMDS_SCREEN)
+	  if (kbemu && context == BRL_CTX_SCREEN)
 	    break;
 
 	  /* if it's a dangerous command it should have been chorded */
@@ -614,39 +602,39 @@ brl_readCommand (BrailleDisplay *brl, DriverCommandContext cmds)
 	    if(kbemu)
 	      message ("keyboard emu on", MSG_SILENT);
 	    else message ("keyboard emu off", MSG_SILENT);
-	    return CMD_NOOP;
+	    return BRL_CMD_NOOP;
 	  case BLT_ROTATE:	/* rotate Braille Lite by 180 degrees */
 	    reverse_kbd ^= 1;
-	    return CMD_NOOP;
+	    return BRL_CMD_NOOP;
 	  case BLT_POSITN:	/* position internal cursor */
 	    int_cursor = blitesz / 2;
 	    state = ST_CURSOR;
-	    return CMD_NOOP;
+	    return BRL_CMD_NOOP;
 	  case BLT_REPEAT:	/* set repeat count */
 	    hold = 0;
 	    sprintf (outmsg, "Repeat count:");
 	    message (outmsg, MSG_SILENT | MSG_NODELAY);
 	    intoverride = 1;
 	    state = ST_REPEAT;
-	    return CMD_NOOP;
+	    return BRL_CMD_NOOP;
 	  case BLT_CONFIG:	/* configuration menu */
 	    sprintf (outmsg, "Config? [m/s/r/z]");
 	    message (outmsg, MSG_SILENT | MSG_NODELAY);
 	    intoverride = 1;
 	    state = ST_CONFIG;
-	    return CMD_NOOP;
+	    return BRL_CMD_NOOP;
 	  case ' ':		/* practical exception for */
-	    /* If keyboard mode off, space bar == CMD_HOME */
-	    if (!kbemu || cmds != CMDS_SCREEN)
-	      return CMD_HOME;
+	    /* If keyboard mode off, space bar == BRL_CMD_HOME */
+	    if (!kbemu || context != BRL_CTX_SCREEN)
+	      return BRL_CMD_HOME;
 	  }
 
       /* check for routing keys */
       if (key.routing)
-	return (CR_ROUTE + key.routing - 1);
+	return (BRL_BLK_ROUTE + key.routing - 1);
 
       if (!kbemu)
-	return CMD_NOOP;
+	return BRL_CMD_NOOP;
 
       /* Now kbemu is definitely on. */
       switch (key.raw & 0xC0)
@@ -674,49 +662,49 @@ brl_readCommand (BrailleDisplay *brl, DriverCommandContext cmds)
 	      shiftlck = 1;
 	    else
 	      shift = 1;
-	    return CMD_NOOP;
+	    return BRL_CMD_NOOP;
 	  case BLT_UPCOFF:	/* cancel upper case */
 	    shift = shiftlck = 0;
-	    return CMD_NOOP;
+	    return BRL_CMD_NOOP;
 	  case BLT_CTRL:	/* control next */
 	    ctrl = 1;
-	    return CMD_NOOP;
+	    return BRL_CMD_NOOP;
 #ifdef USE_TEXTTRANS
 	  case BLT_DOT8SHIFT:	/* add dot 8 to next pattern */
 	    dot8shift = 1;
-	    return CMD_NOOP;
+	    return BRL_CMD_NOOP;
 #endif /* USE_TEXTTRANS */
 	  case BLT_META:	/* meta next */
 	    meta = 1;
-	    return CMD_NOOP;
+	    return BRL_CMD_NOOP;
 	  case BLT_ABORT:	/* abort - quit keyboard emulation */
 	    kbemu = 0;
 	    message ("keyboard emu off", MSG_SILENT);
-	    return CMD_NOOP;
+	    return BRL_CMD_NOOP;
 	  default:		/* unrecognised command */
 	    shift = shiftlck = ctrl = meta = 0;
 #ifdef USE_TEXTTRANS
 	    dot8shift = 0;
 #endif /* USE_TEXTTRANS */
-	    return CMD_NOOP;
+	    return BRL_CMD_NOOP;
 	  }
 
       /* OK, it's an ordinary (non-chorded) keystroke, and kbemu is on. */
 #ifndef USE_TEXTTRANS
       if (ctrl && key.asc >= 96)
 	/* old code was (key.asc & 0x1f) */
-	temp = VAL_PASSCHAR | key.asc | VPC_CONTROL;
+	temp = BRL_BLK_PASSCHAR | key.asc | BRL_FLG_CHAR_CONTROL;
       else if (meta && key.asc >= 96)
-	temp = VAL_PASSCHAR | key.asc | VPC_META;
+	temp = BRL_BLK_PASSCHAR | key.asc | BRL_FLG_CHAR_META;
       else if (shift && (key.asc & 0x40))
 	/* old code was (key.asc & 0xdf) */
-	temp = VAL_PASSCHAR | key.asc | VPC_SHIFT;
+	temp = BRL_BLK_PASSCHAR | key.asc | BRL_FLG_CHAR_SHIFT;
       else
-	temp = VAL_PASSCHAR | key.asc;
+	temp = BRL_BLK_PASSCHAR | key.asc;
 #else /* USE_TEXTTRANS */
-      temp = VAL_PASSDOTS |
+      temp = BRL_BLK_PASSDOTS |
 	(keys_to_dots[key.raw &0x3F]
-	 | ((meta) ? VPC_META : 0)
+	 | ((meta) ? BRL_FLG_CHAR_META : 0)
 	 | ((ctrl) ? 0xC0 : 
 	    (shift) ? 0x40 : 
 	    (dot8shift) ? 0x80 : 0));
@@ -733,39 +721,39 @@ brl_readCommand (BrailleDisplay *brl, DriverCommandContext cmds)
     case ST_CURSOR:			/* position internal cursor */
       switch (key.cmd)
 	{
-	case CMD_HOME:		/* go to middle */
+	case BRL_CMD_HOME:		/* go to middle */
 	  int_cursor = blitesz / 2;
 	  break;
-	case CMD_LNBEG:	/* beginning of display */
+	case BRL_CMD_LNBEG:	/* beginning of display */
 	  int_cursor = 1;
 	  break;
-	case CMD_LNEND:	/* end of display */
+	case BRL_CMD_LNEND:	/* end of display */
 	  int_cursor = blitesz;
 	  break;
-	case CMD_FWINLT:	/* quarter left */
+	case BRL_CMD_FWINLT:	/* quarter left */
 	  int_cursor = MAX (int_cursor - blitesz / 4, 1);
 	  break;
-	case CMD_FWINRT:	/* quarter right */
+	case BRL_CMD_FWINRT:	/* quarter right */
 	  int_cursor = MIN (int_cursor + blitesz / 4, blitesz);
 	  break;
-	case CMD_CHRLT:	/* one character left */
+	case BRL_CMD_CHRLT:	/* one character left */
 	  if (int_cursor > 1)
 	    int_cursor--;
 	  break;
-	case CMD_CHRRT:	/* one character right */
+	case BRL_CMD_CHRRT:	/* one character right */
 	  if (int_cursor < blitesz)
 	    int_cursor++;
 	  break;
-	case CR_ROUTE:	/* route cursor */
+	case BRL_BLK_ROUTE:	/* route cursor */
 	  if (key.spcbar)
 	    {
-	      temp = CR_ROUTE + int_cursor - 1;
+	      temp = BRL_BLK_ROUTE + int_cursor - 1;
 	      int_cursor = 0;
 	      state = ST_NORMAL;
 	    }
 	  return temp;
-	case CR_CUTBEGIN:	/* begin cut */
-	case CR_CUTAPPEND:
+	case BRL_BLK_CUTBEGIN:	/* begin cut */
+	case BRL_BLK_CUTAPPEND:
 	  if (key.spcbar)
 	    {
 	      temp = key.cmd + int_cursor - 1;
@@ -773,8 +761,8 @@ brl_readCommand (BrailleDisplay *brl, DriverCommandContext cmds)
 	      state = ST_NORMAL;
 	    }
 	  return temp;
-	case CR_CUTRECT:	/* end cut */
-	case CR_CUTLINE:
+	case BRL_BLK_CUTRECT:	/* end cut */
+	case BRL_BLK_CUTLINE:
 	  if (key.spcbar)
 	    {
 	      temp = key.cmd + int_cursor - 1;
@@ -782,8 +770,8 @@ brl_readCommand (BrailleDisplay *brl, DriverCommandContext cmds)
 	      state = ST_NORMAL;
 	    }
 	  return temp;
-	case CMD_DISPMD: /* attribute info */
-	  temp = CR_DESCCHAR + int_cursor - 1;
+	case BRL_CMD_DISPMD: /* attribute info */
+	  temp = BRL_BLK_DESCCHAR + int_cursor - 1;
 	  int_cursor = 0;
 	  state = ST_NORMAL;
 	  return temp;
@@ -797,7 +785,7 @@ brl_readCommand (BrailleDisplay *brl, DriverCommandContext cmds)
 	}
       if (key.routing)
 	int_cursor = key.routing;
-      return CMD_NOOP;
+      return BRL_CMD_NOOP;
     case ST_REPEAT:			/* set repeat count */
       if (key.asc >= '0' && key.asc <= '9')
 	{
@@ -824,11 +812,11 @@ brl_readCommand (BrailleDisplay *brl, DriverCommandContext cmds)
 	if (hold > 0) {
 	  if (key.asc == SWITCHVT_NEXT || key.asc == SWITCHVT_PREV)
 	    /* That's chorded or not... */
-	    return CR_SWITCHVT + (hold-1);
+	    return BRL_BLK_SWITCHVT + (hold-1);
 	  else if (key.asc == O_SETMARK)
-	    return CR_SETMARK + (hold-1);
+	    return BRL_BLK_SETMARK + (hold-1);
 	  else if (key.asc == O_GOTOMARK)
-	    return CR_GOTOMARK + (hold-1);
+	    return BRL_BLK_GOTOMARK + (hold-1);
 	  else if (key.spcbar)		/* chorded */
 	    switch (key.asc)
 	      {
@@ -840,34 +828,34 @@ brl_readCommand (BrailleDisplay *brl, DriverCommandContext cmds)
 		}
 		/* fall through */
 	      case BLT_ABORT:	/* abort or endcmd */
-		return CMD_NOOP;
+		return BRL_CMD_NOOP;
 	      }
 	  /* if the key is any other, start repeating it. */
 	  repeat = hold;
 	  goto again;
 	}
       }
-      return CMD_NOOP;
+      return BRL_CMD_NOOP;
     case ST_CONFIG:			/* preferences options */
       switch (key.asc)
 	{
 	case 'm':		/* preferences menu */
 	  intoverride = 0;
 	  state = ST_NORMAL;
-	  return CMD_PREFMENU;
+	  return BRL_CMD_PREFMENU;
 	case 's':		/* save preferences */
 	  intoverride = 0;
 	  state = ST_NORMAL;
-	  return CMD_PREFSAVE;
+	  return BRL_CMD_PREFSAVE;
 	case 'r':		/* restore saved preferences */
 	  intoverride = 0;
 	  state = ST_NORMAL;
-	  return CMD_PREFLOAD;
+	  return BRL_CMD_PREFLOAD;
 	case BLT_ABORT:	/* abort */
 	  intoverride = 0;
 	  state = ST_NORMAL;
 	default:		/* in any case */
-	  return CMD_NOOP;
+	  return BRL_CMD_NOOP;
 	}
     }
 
