@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the Linux console (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2003 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -21,14 +21,14 @@
  * Edit as necessary for your system.
  */
 
+/* used by speech.c */
+#include "Programs/serial.h"
+extern SerialDevice *BL_serialDevice;
+
 /* We always expect 8 data bits, no parity, 1 stop bit. */
 /* Select baudrate to use */
-#define BAUDRATE B9600
-//#define BAUDRATE B38400
-
-/* Define this if you want to keep trying to detect a display forever,
-   instead of exiting when no display appears to be connected. */
-#define DETECT_FOREVER
+#define BAUDRATE 9600
+//#define BAUDRATE 38400
 
 /* Define the following for dots to character mapping for input to use 
    the same (user-defined) table as is used for output, instead of the
