@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the Linux console (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2003 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -30,7 +30,6 @@
 #include <string.h>
 #include <fcntl.h>
 #include <sys/ioctl.h>
-#include <sys/termios.h>
 
 #ifdef HAVE_LINUX_VT_H
 #include <linux/vt.h>
@@ -46,11 +45,11 @@ typedef enum {
 #define BRLPARMS "statuscells"
 
 #include "Programs/brl_driver.h"
+#include "Programs/tbl.h"
 #include "braille.h"
+#include "Programs/serial.h"
 
-static int fileDescriptor = -1;
-static struct termios oldSettings;
-static struct termios newSettings;
+static SerialDevice *serialDevice = NULL;
 
 static unsigned char *cellBuffer = NULL;
 static unsigned int cellCount = 0;
@@ -92,9 +91,7 @@ refreshCells (void) {
       }
    }
 
-   if (safe_write(fileDescriptor, outputBuffer, output-outputBuffer) == -1) {
-      LogError("BrailleNote write");
-   }
+   serialWriteData(serialDevice, outputBuffer, output-outputBuffer);
 }
 
 static void
@@ -117,11 +114,8 @@ writePrompt (const unsigned char *prompt) {
 static unsigned char
 getByte (void) {
    unsigned char byte;
-   fd_set mask;
-   FD_ZERO(&mask);
-   FD_SET(fileDescriptor, &mask);
-   select(fileDescriptor+1, &mask, NULL, NULL, NULL);
-   read(fileDescriptor, &byte, 1);
+   while (!serialAwaitInput(serialDevice, 1000000000));
+   serialReadData(serialDevice, &byte, 1, 0, 0);
    return byte;
 }
 
@@ -237,7 +231,7 @@ writeVisualDisplay (unsigned char c) {
 }
 
 static int
-visualDisplay (unsigned char character, DriverCommandContext cmds) {
+visualDisplay (unsigned char character, BRL_DriverCommandContext context) {
    int vt = getVirtualTerminal();
    const unsigned char end[] = {0X1B, 0};
    unsigned int state = 0;
@@ -279,7 +273,7 @@ static void
 adjustStatusCells (BrailleDisplay *brl, const char *parameter) {
    if (*parameter) {
       const int minimum = 0;
-      const int maximum = MIN(StatusCellCount, brl->x-1);
+      const int maximum = MIN(BRL_MAX_STATUS_CELL_COUNT, brl->x-1);
       if (validateInteger(&statusCells, "status cell count", parameter, &minimum, &maximum)) {
          statusArea = dataArea;
 	 dataArea = statusArea + statusCells;
@@ -297,69 +291,66 @@ brl_open (BrailleDisplay *brl, char **parameters, const char *device) {
       reverseTranslationTable(&outputTable, &inputTable);
    }
 
-   if (openSerialDevice(device, &fileDescriptor, &oldSettings)) {
-      memset(&newSettings, 0, sizeof(newSettings));
-      newSettings.c_cflag = CS8 | CSTOPB | CLOCAL | CREAD;
-      newSettings.c_iflag = IGNPAR;
-      while (resetSerialDevice(fileDescriptor, &newSettings, B38400)) {
+   if (!isSerialDevice(&device)) {
+      unsupportedDevice(device);
+      return 0;
+   }
+
+   if ((serialDevice = serialOpenDevice(device))) {
+      while (serialRestartDevice(serialDevice, 38400)) {
          unsigned char request[] = {BNO_BEGIN, BNO_DESCRIBE};
-         if (safe_write(fileDescriptor, request, sizeof(request)) != -1) {
-            if (awaitInput(fileDescriptor, 1000)) {
-               unsigned char response[3];
-               int offset = 0;
-               if (readChunk(fileDescriptor, response, &offset, sizeof(response), 100)) {
-                  if (response[0] == BNI_DESCRIBE) {
-                     statusCells = response[1];
-                     brl->x = response[2];
-                     brl->y = 1;
-                     if ((statusCells == 5) && (brl->x == 30)) {
-                        statusCells -= 2;
-                        brl->x += 2;
-                     }
-                     dataCells = brl->x * brl->y;
-                     cellCount = statusCells + dataCells;
-                     if ((cellBuffer = malloc(cellCount))) {
-                        memset(cellBuffer, 0, cellCount);
-                        statusArea = cellBuffer;
-                        dataArea = statusArea + statusCells;
-                        if ((outputBuffer = malloc(2 + (cellCount * 2)))) {
-                           refreshCells();
-                           persistentKeyboardMode = KBM_NAVIGATE;
-                           temporaryKeyboardMode = persistentKeyboardMode;
-                           persistentRoutingOperation = CR_ROUTE;
-                           temporaryRoutingOperation = persistentRoutingOperation;
-                           adjustStatusCells(brl, parameters[PARM_STATUSCELLS]);
-                           return 1;
-                        } else {
-                           LogError("Output buffer allocation");
-                        }
-                        free(cellBuffer);
-                     } else {
-                        LogError("Cell buffer allocation");
-                     }
-                  } else {
-                     LogPrint(LOG_ERR, "Unexpected BrailleNote description: %02X %02X %02X",
-                              response[0], response[1], response[2]);
+         if (serialWriteData(serialDevice, request, sizeof(request)) != -1) {
+            unsigned char response[3];
+            if (serialReadData(serialDevice, response, sizeof(response), 1000, 100) == sizeof(response)) {
+               if (response[0] == BNI_DESCRIBE) {
+                  statusCells = response[1];
+                  brl->x = response[2];
+                  brl->y = 1;
+                  if ((statusCells == 5) && (brl->x == 30)) {
+                     statusCells -= 2;
+                     brl->x += 2;
                   }
+                  dataCells = brl->x * brl->y;
+                  cellCount = statusCells + dataCells;
+                  if ((cellBuffer = malloc(cellCount))) {
+                     memset(cellBuffer, 0, cellCount);
+                     statusArea = cellBuffer;
+                     dataArea = statusArea + statusCells;
+                     if ((outputBuffer = malloc(2 + (cellCount * 2)))) {
+                        refreshCells();
+                        persistentKeyboardMode = KBM_NAVIGATE;
+                        temporaryKeyboardMode = persistentKeyboardMode;
+                        persistentRoutingOperation = BRL_BLK_ROUTE;
+                        temporaryRoutingOperation = persistentRoutingOperation;
+                        adjustStatusCells(brl, parameters[PARM_STATUSCELLS]);
+                        return 1;
+                     } else {
+                        LogError("Output buffer allocation");
+                     }
+                     free(cellBuffer);
+                  } else {
+                     LogError("Cell buffer allocation");
+                  }
+               } else {
+                  LogPrint(LOG_ERR, "Unexpected BrailleNote description: %02X %02X %02X",
+                           response[0], response[1], response[2]);
                }
             }
          } else {
             LogError("Write");
          }
-         delay(1000);
+         approximateDelay(1000);
       }
-      tcsetattr(fileDescriptor, TCSANOW, &oldSettings);
-      close(fileDescriptor);
-      fileDescriptor = -1;
+      serialCloseDevice(serialDevice);
+      serialDevice = NULL;
    }
    return 0;
 }
 
 static void
 brl_close (BrailleDisplay *brl) {
-   tcsetattr(fileDescriptor, TCSADRAIN, &oldSettings);
-   close(fileDescriptor);
-   fileDescriptor = -1;
+   serialCloseDevice(serialDevice);
+   serialDevice = NULL;
 
    free(outputBuffer);
    outputBuffer = NULL;
@@ -493,88 +484,88 @@ static int
 getFunctionKey (void) {
    unsigned int keyNumber;
    if (getDecimalInteger(&keyNumber, 2, "function key")) {
-      if (!keyNumber) keyNumber = 0X100 - VPK_FUNCTION;
-      return VAL_PASSKEY + VPK_FUNCTION + (keyNumber - 1);
+      if (!keyNumber) keyNumber = 0X100 - BRL_KEY_FUNCTION;
+      return BRL_BLK_PASSKEY + BRL_KEY_FUNCTION + (keyNumber - 1);
    }
    return EOF;
 }
 
 static int
-interpretNavigation (unsigned char dots, DriverCommandContext cmds) {
+interpretNavigation (unsigned char dots, BRL_DriverCommandContext context) {
    switch (dots) {
       default:
          break;
       case BND_0:
-         return CMD_HOME;
+         return BRL_CMD_HOME;
       case (BND_1):
-	 return CMD_CHRLT;
+	 return BRL_CMD_CHRLT;
       case (BND_1 | BND_2):
-	 return CMD_HWINLT;
+	 return BRL_CMD_HWINLT;
       case (BND_2):
-	 return CMD_FWINLT;
+	 return BRL_CMD_FWINLT;
       case (BND_2 | BND_3):
-	 return CMD_FWINLTSKIP;
+	 return BRL_CMD_FWINLTSKIP;
       case (BND_3):
-	 return CMD_LNBEG;
+	 return BRL_CMD_LNBEG;
       case (BND_1 | BND_3):
-	 return CMD_LNUP;
+	 return BRL_CMD_LNUP;
       case (BND_1 | BND_2 | BND_3):
-	 return CMD_TOP_LEFT;
+	 return BRL_CMD_TOP_LEFT;
       case (BND_4):
-	 return CMD_CHRRT;
+	 return BRL_CMD_CHRRT;
       case (BND_4 | BND_5):
-	 return CMD_HWINRT;
+	 return BRL_CMD_HWINRT;
       case (BND_5):
-	 return CMD_FWINRT;
+	 return BRL_CMD_FWINRT;
       case (BND_5 | BND_6):
-	 return CMD_FWINRTSKIP;
+	 return BRL_CMD_FWINRTSKIP;
       case (BND_6):
-	 return CMD_LNEND;
+	 return BRL_CMD_LNEND;
       case (BND_4 | BND_6):
-	 return CMD_LNDN;
+	 return BRL_CMD_LNDN;
       case (BND_4 | BND_5 | BND_6):
-	 return CMD_BOT_LEFT;
+	 return BRL_CMD_BOT_LEFT;
       case (BND_1 | BND_4):
-	 return CMD_TOP;
+	 return BRL_CMD_TOP;
       case (BND_2 | BND_5):
-	 return CMD_HOME;
+	 return BRL_CMD_HOME;
       case (BND_3 | BND_6):
-	 return CMD_BOT;
+	 return BRL_CMD_BOT;
       case (BND_1 | BND_4 | BND_5):
-	 return CMD_PRDIFLN;
+	 return BRL_CMD_PRDIFLN;
       case (BND_2 | BND_5 | BND_6):
-	 return CMD_NXDIFLN;
+	 return BRL_CMD_NXDIFLN;
       case (BND_1 | BND_2 | BND_4):
-	 return CMD_PRSEARCH;
+	 return BRL_CMD_PRSEARCH;
       case (BND_2 | BND_3 | BND_5):
-	 return CMD_NXSEARCH;
+	 return BRL_CMD_NXSEARCH;
       case (BND_1 | BND_2 | BND_5):
-	 return CMD_ATTRUP;
+	 return BRL_CMD_ATTRUP;
       case (BND_2 | BND_3 | BND_6):
-	 return CMD_ATTRDN;
+	 return BRL_CMD_ATTRDN;
       case (BND_2 | BND_4):
-	 temporaryRoutingOperation = CR_PRINDENT;
-	 return CMD_NOOP;
+	 temporaryRoutingOperation = BRL_BLK_PRINDENT;
+	 return BRL_CMD_NOOP;
       case (BND_3 | BND_5):
-	 temporaryRoutingOperation = CR_NXINDENT;
-	 return CMD_NOOP;
+	 temporaryRoutingOperation = BRL_BLK_NXINDENT;
+	 return BRL_CMD_NOOP;
       case (BND_2 | BND_4 | BND_5):
-	 return CMD_WINUP;
+	 return BRL_CMD_WINUP;
       case (BND_3 | BND_5 | BND_6):
-	 return CMD_WINDN;
+	 return BRL_CMD_WINDN;
    }
    return EOF;
 }
 
 static int
-interpretCharacter (unsigned char dots, DriverCommandContext cmds) {
+interpretCharacter (unsigned char dots, BRL_DriverCommandContext context) {
    int mask = 0X00;
-   if (cmds != CMDS_SCREEN) {
-      return interpretNavigation(dots, cmds);
+   if (context != BRL_CTX_SCREEN) {
+      return interpretNavigation(dots, context);
    }
    switch (currentKeyboardMode) {
       case KBM_NAVIGATE:
-         return interpretNavigation(dots, cmds);
+         return interpretNavigation(dots, context);
       case KBM_INPUT:
 	 break;
       case KBM_INPUT_7:
@@ -587,11 +578,11 @@ interpretCharacter (unsigned char dots, DriverCommandContext cmds) {
 	 mask |= 0X80;
 	 break;
    }
-   return VAL_PASSDOTS + (inputTable[dots] | mask);
+   return BRL_BLK_PASSDOTS + (inputTable[dots] | mask);
 }
 
 static int
-interpretSpaceChord (unsigned char dots, DriverCommandContext cmds) {
+interpretSpaceChord (unsigned char dots, BRL_DriverCommandContext context) {
    switch (dots) {
       default:
       /* These are overridden by the Braille Note itself. */
@@ -606,83 +597,83 @@ interpretSpaceChord (unsigned char dots, DriverCommandContext cmds) {
       case (BND_1 | BND_2 | BND_3 | BND_4 | BND_5 | BND_6): /* go to main menu */
 	 break;
       case BNC_SPACE:
-	 return interpretCharacter(dots, cmds);
+	 return interpretCharacter(dots, context);
       case BNC_C:
-	 return CMD_PREFMENU;
+	 return BRL_CMD_PREFMENU;
       case BNC_D:
-	 return CMD_PREFLOAD;
+	 return BRL_CMD_PREFLOAD;
       case BNC_F:
 	 return getFunctionKey();
       case BNC_L:
-	 temporaryRoutingOperation = CR_SETLEFT;
-	 return CMD_NOOP;
+	 temporaryRoutingOperation = BRL_BLK_SETLEFT;
+	 return BRL_CMD_NOOP;
       case BNC_M:
-	 return CMD_MUTE;
+	 return BRL_CMD_MUTE;
       case BNC_N:
          persistentKeyboardMode = KBM_NAVIGATE;
 	 temporaryKeyboardMode = persistentKeyboardMode;
-	 return CMD_NOOP;
+	 return BRL_CMD_NOOP;
       case BNC_P:
-	 return CMD_PASTE;
+	 return BRL_CMD_PASTE;
       case BNC_S:
-	 return CMD_SAY_LINE;
+	 return BRL_CMD_SAY_LINE;
       case BNC_V: {
 	 unsigned int vt;
 	 if (getDecimalInteger(&vt, 2, "virt term num")) {
 	    if (!vt) vt = 0X100;
-	    return CR_SWITCHVT + (vt - 1);
+	    return BRL_BLK_SWITCHVT + (vt - 1);
 	 }
          return EOF;
       }
       case BNC_W:
-	 return CMD_PREFSAVE;
+	 return BRL_CMD_PREFSAVE;
       case BNC_X: {
 	 unsigned char character;
 	 if (!getHexadecimalCharacter(&character)) {
 	    return EOF;
 	 }
-	 return VAL_PASSCHAR + character;
+	 return BRL_BLK_PASSCHAR + character;
       }
       case BNC_LPAREN:
-	 temporaryRoutingOperation = CR_CUTBEGIN;
-	 return CMD_NOOP;
+	 temporaryRoutingOperation = BRL_BLK_CUTBEGIN;
+	 return BRL_CMD_NOOP;
       case BNC_LBRACE:
-	 temporaryRoutingOperation = CR_CUTAPPEND;
-	 return CMD_NOOP;
+	 temporaryRoutingOperation = BRL_BLK_CUTAPPEND;
+	 return BRL_CMD_NOOP;
       case BNC_RPAREN:
-	 temporaryRoutingOperation = CR_CUTRECT;
-	 return CMD_NOOP;
+	 temporaryRoutingOperation = BRL_BLK_CUTRECT;
+	 return BRL_CMD_NOOP;
       case BNC_RBRACE:
-	 temporaryRoutingOperation = CR_CUTLINE;
-	 return CMD_NOOP;
+	 temporaryRoutingOperation = BRL_BLK_CUTLINE;
+	 return BRL_CMD_NOOP;
       case BNC_BAR:
-         return CMD_CSRJMP_VERT;
+         return BRL_CMD_CSRJMP_VERT;
       case BNC_QUESTION:
-         return CMD_LEARN;
+         return BRL_CMD_LEARN;
       case (BND_2 | BND_3 | BND_5 | BND_6):
-	 return VAL_PASSKEY + VPK_TAB;
+	 return BRL_BLK_PASSKEY + BRL_KEY_TAB;
       case (BND_2 | BND_3):
-	 return VAL_PASSKEY + VPK_CURSOR_LEFT;
+	 return BRL_BLK_PASSKEY + BRL_KEY_CURSOR_LEFT;
       case (BND_5 | BND_6):
-	 return VAL_PASSKEY + VPK_CURSOR_RIGHT;
+	 return BRL_BLK_PASSKEY + BRL_KEY_CURSOR_RIGHT;
       case (BND_2 | BND_5):
-	 return VAL_PASSKEY + VPK_CURSOR_UP;
+	 return BRL_BLK_PASSKEY + BRL_KEY_CURSOR_UP;
       case (BND_3 | BND_6):
-	 return VAL_PASSKEY + VPK_CURSOR_DOWN;
+	 return BRL_BLK_PASSKEY + BRL_KEY_CURSOR_DOWN;
       case (BND_2):
-	 return VAL_PASSKEY + VPK_HOME;
+	 return BRL_BLK_PASSKEY + BRL_KEY_HOME;
       case (BND_3):
-	 return VAL_PASSKEY + VPK_END;
+	 return BRL_BLK_PASSKEY + BRL_KEY_END;
       case (BND_5):
-	 return VAL_PASSKEY + VPK_PAGE_UP;
+	 return BRL_BLK_PASSKEY + BRL_KEY_PAGE_UP;
       case (BND_6):
-	 return VAL_PASSKEY + VPK_PAGE_DOWN;
+	 return BRL_BLK_PASSKEY + BRL_KEY_PAGE_DOWN;
       case (BND_3 | BND_5):
-	 return VAL_PASSKEY + VPK_INSERT;
+	 return BRL_BLK_PASSKEY + BRL_KEY_INSERT;
       case (BND_2 | BND_5 | BND_6):
-	 return VAL_PASSKEY + VPK_DELETE;
+	 return BRL_BLK_PASSKEY + BRL_KEY_DELETE;
       case (BND_2 | BND_6):
-	 return VAL_PASSKEY + VPK_ESCAPE;
+	 return BRL_BLK_PASSKEY + BRL_KEY_ESCAPE;
       case (BND_4):
       case (BND_4 | BND_5):
          temporaryKeyboardMode = KBM_INPUT;
@@ -702,55 +693,55 @@ interpretSpaceChord (unsigned char dots, DriverCommandContext cmds) {
 	 if (dots & BND_5) {
 	    persistentKeyboardMode = temporaryKeyboardMode;
 	 }
-	 return CMD_NOOP;
+	 return BRL_CMD_NOOP;
    }
    return EOF;
 }
 
 static int
-interpretBackspaceChord (unsigned char dots, DriverCommandContext cmds) {
+interpretBackspaceChord (unsigned char dots, BRL_DriverCommandContext context) {
    switch (dots & 0X3F) {
       default:
 	 break;
       case BNC_SPACE:
-	 return VAL_PASSKEY + VPK_BACKSPACE;
+	 return BRL_BLK_PASSKEY + BRL_KEY_BACKSPACE;
 	 return EOF;
       case BNC_A:
-	 return CMD_DISPMD | VAL_SWITCHON;
+	 return BRL_CMD_DISPMD | BRL_FLG_TOGGLE_ON;
       case BNC_B:
-         return CMD_SKPBLNKWINS | VAL_SWITCHOFF;
+         return BRL_CMD_SKPBLNKWINS | BRL_FLG_TOGGLE_OFF;
       case BNC_D:
-	 temporaryRoutingOperation = CR_DESCCHAR;
-         return CMD_NOOP;
+	 temporaryRoutingOperation = BRL_BLK_DESCCHAR;
+         return BRL_CMD_NOOP;
       case BNC_F:
-         return CMD_FREEZE | VAL_SWITCHOFF;
+         return BRL_CMD_FREEZE | BRL_FLG_TOGGLE_OFF;
       case BNC_H:
-	 return CMD_HELP;
+	 return BRL_CMD_HELP;
       case BNC_I:
-         return CMD_SKPIDLNS | VAL_SWITCHOFF;
+         return BRL_CMD_SKPIDLNS | BRL_FLG_TOGGLE_OFF;
       case BNC_M:
-	 temporaryRoutingOperation = CR_SETMARK;
-	 return CMD_NOOP;
+	 temporaryRoutingOperation = BRL_BLK_SETMARK;
+	 return BRL_CMD_NOOP;
       case BNC_S:
-	 return CMD_INFO;
+	 return BRL_CMD_INFO;
       case BNC_T:
-	 return CMD_DISPMD | VAL_SWITCHOFF;
+	 return BRL_CMD_DISPMD | BRL_FLG_TOGGLE_OFF;
       case BNC_V:
-	 return CMD_SWITCHVT_PREV;
+	 return BRL_CMD_SWITCHVT_PREV;
       case BNC_W:
-         return CMD_SLIDEWIN | VAL_SWITCHOFF;
+         return BRL_CMD_SLIDEWIN | BRL_FLG_TOGGLE_OFF;
       case BNC_6:
-         return CMD_SIXDOTS | VAL_SWITCHON;
+         return BRL_CMD_SIXDOTS | BRL_FLG_TOGGLE_ON;
       case BNC_8:
-         return CMD_SIXDOTS | VAL_SWITCHOFF;
+         return BRL_CMD_SIXDOTS | BRL_FLG_TOGGLE_OFF;
       case (BND_1 | BND_2 | BND_3 | BND_4 | BND_5 | BND_6):
-	 return CMD_RESTARTSPEECH;
+	 return BRL_CMD_RESTARTSPEECH;
    }
    return EOF;
 }
 
 static int
-interpretEnterChord (unsigned char dots, DriverCommandContext cmds) {
+interpretEnterChord (unsigned char dots, BRL_DriverCommandContext context) {
    switch (dots) {
       default:
       /* These are overridden by the Braille Note itself. */
@@ -766,66 +757,66 @@ interpretEnterChord (unsigned char dots, DriverCommandContext cmds) {
       case BNC_T: /* display the time */
 	 break;
       case BNC_SPACE:
-	 return VAL_PASSKEY + VPK_RETURN;
+	 return BRL_BLK_PASSKEY + BRL_KEY_ENTER;
 	 return EOF;
       case BNC_B:
-         return CMD_SKPBLNKWINS | VAL_SWITCHON;
+         return BRL_CMD_SKPBLNKWINS | BRL_FLG_TOGGLE_ON;
       case BNC_F:
-         return CMD_FREEZE | VAL_SWITCHON;
+         return BRL_CMD_FREEZE | BRL_FLG_TOGGLE_ON;
       case BNC_I:
-         return CMD_SKPIDLNS | VAL_SWITCHON;
+         return BRL_CMD_SKPIDLNS | BRL_FLG_TOGGLE_ON;
       case BNC_M:
-	 temporaryRoutingOperation = CR_GOTOMARK;
-	 return CMD_NOOP;
+	 temporaryRoutingOperation = BRL_BLK_GOTOMARK;
+	 return BRL_CMD_NOOP;
       case BNC_V:
-	 return CMD_SWITCHVT_NEXT;
+	 return BRL_CMD_SWITCHVT_NEXT;
       case BNC_W:
-         return CMD_SLIDEWIN | VAL_SWITCHON;
+         return BRL_CMD_SLIDEWIN | BRL_FLG_TOGGLE_ON;
       case (BND_1 | BND_2 | BND_3 | BND_4 | BND_5 | BND_6):
-	 return CMD_RESTARTBRL;
+	 return BRL_CMD_RESTARTBRL;
    }
    return EOF;
 }
 
 static int
-interpretThumbKeys (unsigned char keys, DriverCommandContext cmds) {
+interpretThumbKeys (unsigned char keys, BRL_DriverCommandContext context) {
    switch (keys) {
       default:
 	 break;
       case (BNT_PREVIOUS):
-	 return CMD_FWINLT;
+	 return BRL_CMD_FWINLT;
       case (BNT_NEXT):
-	 return CMD_FWINRT;
+	 return BRL_CMD_FWINRT;
       case (BNT_BACK):
-	 return CMD_LNUP;
+	 return BRL_CMD_LNUP;
       case (BNT_ADVANCE):
-	 return CMD_LNDN;
+	 return BRL_CMD_LNDN;
       case (BNT_PREVIOUS | BNT_BACK):
-	 return CMD_LNBEG;
+	 return BRL_CMD_LNBEG;
       case (BNT_NEXT | BNT_ADVANCE):
-	 return CMD_LNEND;
+	 return BRL_CMD_LNEND;
       case (BNT_PREVIOUS | BNT_ADVANCE):
-	 return CMD_TOP_LEFT;
+	 return BRL_CMD_TOP_LEFT;
       case (BNT_PREVIOUS | BNT_NEXT):
-	 return CMD_BOT_LEFT;
+	 return BRL_CMD_BOT_LEFT;
       case (BNT_BACK | BNT_ADVANCE):
-	 return CMD_BACK;
+	 return BRL_CMD_BACK;
       case (BNT_BACK | BNT_NEXT):
-	 return CMD_CSRTRK;
+	 return BRL_CMD_CSRTRK;
    }
    return EOF;
 }
 
 static int
-interpretRoutingKey (unsigned char key, DriverCommandContext cmds) {
+interpretRoutingKey (unsigned char key, BRL_DriverCommandContext context) {
    return currentRoutingOperation + key;
 }
 
 static int
-brl_readCommand (BrailleDisplay *brl, DriverCommandContext cmds) {
+brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
    unsigned char character;
-   int count = read(fileDescriptor, &character, 1);
-   int (*handler)(unsigned char, DriverCommandContext);
+   int count = serialReadData(serialDevice, &character, 1, 0, 0);
+   int (*handler)(unsigned char, BRL_DriverCommandContext);
 
    if (count != 1) {
       if (count == -1) {
@@ -859,8 +850,8 @@ brl_readCommand (BrailleDisplay *brl, DriverCommandContext cmds) {
 	 break;
    }
 
-   delay(1);
-   if (read(fileDescriptor, &character, 1) != 1) {
+   approximateDelay(1);
+   if (serialReadData(serialDevice, &character, 1, 0, 0) != 1) {
       return EOF;
    }
 
@@ -870,5 +861,5 @@ brl_readCommand (BrailleDisplay *brl, DriverCommandContext cmds) {
    currentRoutingOperation = temporaryRoutingOperation;
    temporaryRoutingOperation = persistentRoutingOperation;
 
-   return handler(character, cmds);
+   return handler(character, context);
 }
