@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the Linux console (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2003 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -35,7 +35,7 @@
 /* serial line baudrate... 
  * Note that default braille device is defined in ../Makefile
  */
-#define BAUDRATE B19200
+#define BAUDRATE 19200
 
 /* typematic settings */
 #define TYPEMATIC_DELAY 10	/* nbr of cycles before a key is repeated */
