@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the Linux console (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2003 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -20,7 +20,7 @@
 #include "Programs/brl.h"
 
 
-#define BAUDRATE	B9600
+#define BAUDRATE	9600
 #define CNTX_INTERNAL	0x80
 
 /*
@@ -44,3 +44,6 @@ static int	begblk(BrailleDisplay *);
 static int	endblk(BrailleDisplay *);
 static int	Program(BrailleDisplay *brl);
 static int	ViewOn(BrailleDisplay *);
+static int	routing(BrailleDisplay *brl, int key);
+static int	enter_routing(BrailleDisplay *brl);
+static int	routing(BrailleDisplay *brl, int key);
