@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the Linux console (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2003 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -32,19 +32,15 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <sys/stat.h>
-#include <sys/ioctl.h>
-#include <sys/termios.h>
 
-#include "Programs/brl.h"
 #include "Programs/misc.h"
 
 #define BRLSTAT ST_Generic
 #include "Programs/brl_driver.h"
 #include "braille.h"
+#include "Programs/serial.h"
 
-static int fileDescriptor = -1;
-static struct termios oldSettings;
-static struct termios newSettings;
+static SerialDevice *serialDevice = NULL;
 
 #define screenHeight 25
 #define screenWidth 80
@@ -61,7 +57,7 @@ typedef enum {
 } DeviceStatus;
 static DeviceStatus deviceStatus;
 
-static DriverCommandContext currentContext;
+static BRL_DriverCommandContext currentContext;
 static unsigned char currentLine;
 static unsigned char cursorRow;
 static unsigned char cursorColumn;
@@ -105,35 +101,31 @@ makeDownloadFifo (void) {
 
 static int
 brl_open (BrailleDisplay *brl, char **parameters, const char *device) {
+   if (!isSerialDevice(&device)) {
+      unsupportedDevice(device);
+      return 0;
+   }
+
    makeDownloadFifo();
-   if (openSerialDevice(device, &fileDescriptor, &oldSettings)) {
-      memset(&newSettings, 0, sizeof(newSettings));
-      newSettings.c_cflag = CS8 | CLOCAL | CREAD;
-      newSettings.c_iflag = IGNPAR;
-      if (resetSerialDevice(fileDescriptor, &newSettings, B9600)) {
-         if (tcflush(fileDescriptor, TCIOFLUSH) != -1) {
-            brl->y = screenHeight;
-            brl->x = screenWidth;
-            brl->buffer = &sourceImage[0][0];
-            memset(sourceImage, 0, sizeof(sourceImage));
-            deviceStatus = DEV_ONLINE;
-            return 1;
-         } else {
-            LogError("LogText flush");
-         }
-         tcsetattr(fileDescriptor, TCSANOW, &oldSettings);
+   if ((serialDevice = serialOpenDevice(device))) {
+      if (serialRestartDevice(serialDevice, 9600)) {
+         brl->y = screenHeight;
+         brl->x = screenWidth;
+         brl->buffer = &sourceImage[0][0];
+         memset(sourceImage, 0, sizeof(sourceImage));
+         deviceStatus = DEV_ONLINE;
+         return 1;
       }
-      close(fileDescriptor);
-      fileDescriptor = -1;
+      serialCloseDevice(serialDevice);
+      serialDevice = NULL;
    }
    return 0;
 }
 
 static void
 brl_close (BrailleDisplay *brl) {
-   tcsetattr(fileDescriptor, TCSADRAIN, &oldSettings);
-   close(fileDescriptor);
-   fileDescriptor = -1;
+   serialCloseDevice(serialDevice);
+   serialDevice = NULL;
 }
 
 static int
@@ -158,7 +150,7 @@ checkData (const unsigned char *data, unsigned int length) {
 
 static int
 sendBytes (const unsigned char *bytes, size_t count) {
-   if (write(fileDescriptor, bytes, count) == -1) {
+   if (serialWriteData(serialDevice, bytes, count) == -1) {
       LogError("LogText write");
       return 0;
    }
@@ -242,35 +234,32 @@ brl_writeWindow (BrailleDisplay *brl) {
 
 static int
 isOnline (void) {
-   int signals;
-   if (ioctl(fileDescriptor, TIOCMGET, &signals) == -1) {
-      LogError("TIOCMGET");
-   } else if (!(signals & TIOCM_DSR)) {
-      if (deviceStatus > DEV_OFFLINE) {
-         deviceStatus = DEV_OFFLINE;
-         LogPrint(LOG_WARNING, "LogText offline.");
-      }
-   } else {
+   int online = serialTestLineDSR(serialDevice);
+   if (online) {
       if (deviceStatus < DEV_ONLINE) {
          deviceStatus = DEV_ONLINE;
          LogPrint(LOG_WARNING, "LogText online.");
       }
-      return 1;
+   } else {
+      if (deviceStatus > DEV_OFFLINE) {
+         deviceStatus = DEV_OFFLINE;
+         LogPrint(LOG_WARNING, "LogText offline.");
+      }
    }
-   return 0;
+   return online;
 }
 
 static void
 brl_writeStatus (BrailleDisplay *brl, const unsigned char *status) {
    if (isOnline()) {
-      if (status[FirstStatusCell] == FSC_GENERIC) {
-         unsigned char row = status[STAT_CSRROW];
-         unsigned char column = status[STAT_CSRCOL];
+      if (status[BRL_firstStatusCell] == BRL_STATUS_CELLS_GENERIC) {
+         unsigned char row = status[BRL_GSC_CSRROW];
+         unsigned char column = status[BRL_GSC_CSRCOL];
          row = MAX(1, MIN(row, screenHeight)) - 1;
          column = MAX(1, MIN(column, screenWidth)) - 1;
          if (deviceStatus < DEV_READY) {
             memset(targetImage, 0, sizeof(targetImage));
-            currentContext = CMDS_SCREEN;
+            currentContext = BRL_CTX_SCREEN;
             currentLine = row;
             cursorRow = screenHeight;
             cursorColumn = screenWidth;
@@ -290,7 +279,7 @@ static int
 readKey (void) {
    unsigned char key;
    unsigned char arg;
-   if (read(fileDescriptor, &key, 1) != 1) return EOF;
+   if (serialReadData(serialDevice, &key, 1, 0, 0) != 1) return EOF;
    switch (key) {
       default:
          arg = 0;
@@ -298,7 +287,7 @@ readKey (void) {
       case KEY_FUNCTION:
       case KEY_FUNCTION2:
       case KEY_UPDATE:
-         while (read(fileDescriptor, &arg, 1) != 1) delay(1);
+         while (serialReadData(serialDevice, &arg, 1, 0, 0) != 1) approximateDelay(1);
          break;
    }
    {
@@ -353,7 +342,7 @@ askUser (const unsigned char *prompt) {
    while (1) {
       int key = readKey();
       if (key == EOF) {
-         delay(1);
+         approximateDelay(1);
          continue;
       }
       if ((key & KEY_MASK) == KEY_UPDATE) {
@@ -493,12 +482,12 @@ downloadFile (void) {
 }
 
 static int
-brl_readCommand (BrailleDisplay *brl, DriverCommandContext cmds) {
+brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
    int key = readKey();
-   if (cmds != currentContext) {
-      LogPrint(LOG_DEBUG, "Context switch: %d -> %d", currentContext, cmds);
-      switch (currentContext = cmds) {
-         case CMDS_SCREEN:
+   if (context != currentContext) {
+      LogPrint(LOG_DEBUG, "Context switch: %d -> %d", currentContext, context);
+      switch (currentContext = context) {
+         case BRL_CTX_SCREEN:
             deviceStatus = DEV_ONLINE;
             break;
          default:
@@ -508,95 +497,95 @@ brl_readCommand (BrailleDisplay *brl, DriverCommandContext cmds) {
    if (key != EOF) {
       switch (key) {
          case KEY_FUNCTION_ENTER:
-            return VAL_PASSKEY + VPK_RETURN;
+            return BRL_BLK_PASSKEY + BRL_KEY_ENTER;
          case KEY_FUNCTION_TAB:
-            return VAL_PASSKEY + VPK_TAB;
+            return BRL_BLK_PASSKEY + BRL_KEY_TAB;
          case KEY_FUNCTION_CURSOR_UP:
-            return VAL_PASSKEY + VPK_CURSOR_UP;
+            return BRL_BLK_PASSKEY + BRL_KEY_CURSOR_UP;
          case KEY_FUNCTION_CURSOR_DOWN:
-            return VAL_PASSKEY + VPK_CURSOR_DOWN;
+            return BRL_BLK_PASSKEY + BRL_KEY_CURSOR_DOWN;
          case KEY_FUNCTION_CURSOR_LEFT:
-            return VAL_PASSKEY + VPK_CURSOR_LEFT;
+            return BRL_BLK_PASSKEY + BRL_KEY_CURSOR_LEFT;
          case KEY_FUNCTION_CURSOR_RIGHT:
-            return VAL_PASSKEY + VPK_CURSOR_RIGHT;
+            return BRL_BLK_PASSKEY + BRL_KEY_CURSOR_RIGHT;
          case KEY_FUNCTION_CURSOR_UP_JUMP:
-            return VAL_PASSKEY + VPK_HOME;
+            return BRL_BLK_PASSKEY + BRL_KEY_HOME;
          case KEY_FUNCTION_CURSOR_DOWN_JUMP:
-            return VAL_PASSKEY + VPK_END;
+            return BRL_BLK_PASSKEY + BRL_KEY_END;
          case KEY_FUNCTION_CURSOR_LEFT_JUMP:
-            return VAL_PASSKEY + VPK_PAGE_UP;
+            return BRL_BLK_PASSKEY + BRL_KEY_PAGE_UP;
          case KEY_FUNCTION_CURSOR_RIGHT_JUMP:
-            return VAL_PASSKEY + VPK_PAGE_DOWN;
+            return BRL_BLK_PASSKEY + BRL_KEY_PAGE_DOWN;
          case KEY_FUNCTION_F1:
-            return VAL_PASSKEY + VPK_FUNCTION + 0;
+            return BRL_BLK_PASSKEY + BRL_KEY_FUNCTION + 0;
          case KEY_FUNCTION_F2:
-            return VAL_PASSKEY + VPK_FUNCTION + 1;
+            return BRL_BLK_PASSKEY + BRL_KEY_FUNCTION + 1;
          case KEY_FUNCTION_F3:
-            return VAL_PASSKEY + VPK_FUNCTION + 2;
+            return BRL_BLK_PASSKEY + BRL_KEY_FUNCTION + 2;
          case KEY_FUNCTION_F4:
-            return VAL_PASSKEY + VPK_FUNCTION + 3;
+            return BRL_BLK_PASSKEY + BRL_KEY_FUNCTION + 3;
          case KEY_FUNCTION_F5:
-            return VAL_PASSKEY + VPK_FUNCTION + 4;
+            return BRL_BLK_PASSKEY + BRL_KEY_FUNCTION + 4;
          case KEY_FUNCTION_F6:
-            return VAL_PASSKEY + VPK_FUNCTION + 5;
+            return BRL_BLK_PASSKEY + BRL_KEY_FUNCTION + 5;
          case KEY_FUNCTION_F7:
-            return VAL_PASSKEY + VPK_FUNCTION + 6;
+            return BRL_BLK_PASSKEY + BRL_KEY_FUNCTION + 6;
          case KEY_FUNCTION_F9:
-            return VAL_PASSKEY + VPK_FUNCTION + 8;
+            return BRL_BLK_PASSKEY + BRL_KEY_FUNCTION + 8;
          case KEY_FUNCTION_F10:
-            return VAL_PASSKEY + VPK_FUNCTION + 9;
+            return BRL_BLK_PASSKEY + BRL_KEY_FUNCTION + 9;
          case KEY_COMMAND: {
             int command;
-            while ((command = readKey()) == EOF) delay(1);
+            while ((command = readKey()) == EOF) approximateDelay(1);
             LogPrint(LOG_DEBUG, "Received command: (0x%2.2X) 0x%4.4X", KEY_COMMAND, command);
             switch (command) {
                case KEY_COMMAND:
                   /* pressing the escape command twice will pass it through */
-                  return VAL_PASSDOTS + inputTable[KEY_COMMAND];
+                  return BRL_BLK_PASSDOTS + inputTable[KEY_COMMAND];
                case KEY_COMMAND_SWITCHVT_PREV:
-                  return CMD_SWITCHVT_PREV;
+                  return BRL_CMD_SWITCHVT_PREV;
                case KEY_COMMAND_SWITCHVT_NEXT:
-                  return CMD_SWITCHVT_NEXT;
+                  return BRL_CMD_SWITCHVT_NEXT;
                case KEY_COMMAND_SWITCHVT_1:
-                  return CR_SWITCHVT + 0;
+                  return BRL_BLK_SWITCHVT + 0;
                case KEY_COMMAND_SWITCHVT_2:
-                  return CR_SWITCHVT + 1;
+                  return BRL_BLK_SWITCHVT + 1;
                case KEY_COMMAND_SWITCHVT_3:
-                  return CR_SWITCHVT + 2;
+                  return BRL_BLK_SWITCHVT + 2;
                case KEY_COMMAND_SWITCHVT_4:
-                  return CR_SWITCHVT + 3;
+                  return BRL_BLK_SWITCHVT + 3;
                case KEY_COMMAND_SWITCHVT_5:
-                  return CR_SWITCHVT + 4;
+                  return BRL_BLK_SWITCHVT + 4;
                case KEY_COMMAND_SWITCHVT_6:
-                  return CR_SWITCHVT + 5;
+                  return BRL_BLK_SWITCHVT + 5;
                case KEY_COMMAND_SWITCHVT_7:
-                  return CR_SWITCHVT + 6;
+                  return BRL_BLK_SWITCHVT + 6;
                case KEY_COMMAND_SWITCHVT_8:
-                  return CR_SWITCHVT + 7;
+                  return BRL_BLK_SWITCHVT + 7;
                case KEY_COMMAND_SWITCHVT_9:
-                  return CR_SWITCHVT + 8;
+                  return BRL_BLK_SWITCHVT + 8;
                case KEY_COMMAND_SWITCHVT_10:
-                  return CR_SWITCHVT + 9;
+                  return BRL_BLK_SWITCHVT + 9;
                case KEY_COMMAND_PAGE_UP:
-                  return VAL_PASSKEY + VPK_PAGE_UP;
+                  return BRL_BLK_PASSKEY + BRL_KEY_PAGE_UP;
                case KEY_COMMAND_PAGE_DOWN:
-                  return VAL_PASSKEY + VPK_PAGE_DOWN;
+                  return BRL_BLK_PASSKEY + BRL_KEY_PAGE_DOWN;
                case KEY_COMMAND_PREFMENU:
                   currentLine = 0;
                   cursorRow = 0;
                   cursorColumn = 31;
                   sendCursorRow();
-                  return CMD_PREFMENU;
+                  return BRL_CMD_PREFMENU;
                case KEY_COMMAND_PREFSAVE:
-                  return CMD_PREFSAVE;
+                  return BRL_CMD_PREFSAVE;
                case KEY_COMMAND_PREFLOAD:
-                  return CMD_PREFLOAD;
+                  return BRL_CMD_PREFLOAD;
                case KEY_COMMAND_FREEZE_ON:
-                  return CMD_FREEZE | VAL_SWITCHON;
+                  return BRL_CMD_FREEZE | BRL_FLG_TOGGLE_ON;
                case KEY_COMMAND_FREEZE_OFF:
-                  return CMD_FREEZE | VAL_SWITCHOFF;
+                  return BRL_CMD_FREEZE | BRL_FLG_TOGGLE_OFF;
                case KEY_COMMAND_RESTARTBRL:
-                  return CMD_RESTARTBRL;
+                  return BRL_CMD_RESTARTBRL;
                case KEY_COMMAND_DOWNLOAD:
                   downloadFile();
                   break;
@@ -617,7 +606,7 @@ brl_readCommand (BrailleDisplay *brl, DriverCommandContext cmds) {
                default: {
                   unsigned char dots = inputTable[key];
                   LogPrint(LOG_DEBUG, "Received character: 0X%2.2X dec=%d dots=%2.2X", key, key, dots);
-                  return VAL_PASSDOTS + dots;
+                  return BRL_BLK_PASSDOTS + dots;
                }
             }
             break;
