@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the Linux console (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2003 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -29,11 +29,10 @@
 #include <errno.h>
  
 #include "misc.h"
-#include "contract.h"
+#include "ctb.h"
 #include "ctb_definitions.h"
 #include "brl.h"
 
-
 typedef struct {
   BYTE length;
   BYTE bytes[0XFF];
@@ -45,9 +44,66 @@ static ContractionTableHeader *tableHeader;
 static ContractionTableOffset tableSize;
 static ContractionTableOffset tableUsed;
 
-static BYTE *opcodesTable;
-static int opcodesSize;
-static int opcodesUsed;
+static const char *const characterClassNames[] = {
+  "space",
+  "letter",
+  "digit",
+  "punctuation",
+  "uppercase",
+  "lowercase",
+  NULL
+};
+struct CharacterClass {
+  struct CharacterClass *next;
+  ContractionTableCharacterAttributes attribute;
+  BYTE length;
+  char name[1];
+};
+static struct CharacterClass *characterClasses;
+static ContractionTableCharacterAttributes characterClassAttribute;
+
+static const char *const opcodeNames[CTO_None] = {
+  "include",
+  "locale",
+
+  "capsign",
+  "begcaps",
+  "endcaps",
+
+  "letsign",
+  "numsign",
+
+  "literal",
+  "replace",
+  "always",
+  "repeated",
+
+  "largesign",
+  "word",
+  "joinword",
+  "lowword",
+  "contraction",
+
+  "sufword",
+  "prfword",
+  "begword",
+  "begmidword",
+  "midword",
+  "midendword",
+  "endword",
+
+  "prepunc",
+  "postpunc",
+
+  "begnum",
+  "midnum",
+  "endnum",
+
+  "class",
+  "after",
+  "before"
+};
+static unsigned char opcodeLengths[CTO_None] = {0};
 
 static const char *originalLocale;
 static int noLocale;
@@ -103,7 +159,7 @@ saveBytes (FileData *data, ContractionTableOffset *offset, const BYTE *bytes, BY
 
 static int
 saveString (FileData *data, ContractionTableOffset *offset, const char *string) {
-  return saveBytes(data, offset, string, strlen(string));
+  return saveBytes(data, offset, (const unsigned char *)string, strlen(string));
 }
 
 static int
@@ -111,15 +167,25 @@ saveByteString (FileData *data, ContractionTableOffset *offset, const ByteString
   return saveBytes(data, offset, string->bytes, string->length);
 }
 
-static int
-addRule (FileData *data, ContractionTableOpcode opcode, ByteString *find, ByteString *replace) {
+static ContractionTableRule *
+addRule (
+  FileData *data,
+  ContractionTableOpcode opcode,
+  ByteString *find,
+  ByteString *replace,
+  ContractionTableCharacterAttributes after,
+  ContractionTableCharacterAttributes before
+) {
   ContractionTableOffset ruleOffset;
   int ruleSize = sizeof(ContractionTableRule) - 1;
   if (find) ruleSize += find->length;
   if (replace) ruleSize += replace->length;
   if (allocateBytes(data, &ruleOffset, ruleSize, __alignof__(ContractionTableRule))) {
     ContractionTableRule *newRule = CTR(tableHeader, ruleOffset);
+
     newRule->opcode = opcode;
+    newRule->after = after;
+    newRule->before = before;
     if (find)
       memcpy(&newRule->findrep[0], &find->bytes[0],
              (newRule->findlen = find->length));
@@ -130,103 +196,107 @@ addRule (FileData *data, ContractionTableOpcode opcode, ByteString *find, ByteSt
              (newRule->replen = replace->length));
     else
       newRule->replen = 0;
-    newRule->next = 0;
 
     /*link new rule into table.*/
-    if (newRule->findlen) {
-      ContractionTableOffset bucket;
+    {
+      ContractionTableOffset *offsetAddress;
 
-      /* first, handle single-character find strings. */
       if (newRule->findlen == 1) {
         ContractionTableCharacter *character = &tableHeader->characters[newRule->findrep[0]];
-        if (newRule->opcode == CTO_Always) {
-          character->always = ruleOffset;
-        }
-        if ((bucket = character->rules)) {
-          ContractionTableRule *currentRule = CTR(tableHeader, bucket);
-          while (currentRule->next)
-            currentRule = CTR(tableHeader, currentRule->next);
-          currentRule->next = ruleOffset;
-        } else {
-          character->rules = ruleOffset;
-        }
+        if (newRule->opcode == CTO_Always) character->always = ruleOffset;
+        offsetAddress = &character->rules;
       } else {
-        /* Now, work through the various cases for multi-byte find strings. */
-        bucket = hash(newRule->findrep);
-
-        /* case 1, start new hash chain. */
-        if (!tableHeader->rules[bucket]) {
-          tableHeader->rules[bucket] = ruleOffset;
-        } else {
-          /* Case 2, longest find string goes at head of chain. */
-          ContractionTableRule *currentRule = CTR(tableHeader, tableHeader->rules[bucket]);
-          if (newRule->findlen > currentRule->findlen) {
-            newRule->next = tableHeader->rules[bucket];
-            tableHeader->rules[bucket] = ruleOffset;
-          } else {
-            /* Case 3, new rule goes somewhere in chain. */
-            while (currentRule->next) { /*loop through chain */
-              ContractionTableRule *previousRule = currentRule;
-              currentRule = CTR(tableHeader, currentRule->next);
-              if (newRule->findlen > currentRule->findlen) {
-                newRule->next = previousRule->next;
-                previousRule->next = ruleOffset;
-                break;
-              }
-            }
-            if (!newRule->next) {
-              /* Case 4, new rule goes at end of chain*/
-              currentRule->next = ruleOffset;
-            }
-          }
-        }
+        offsetAddress = &tableHeader->rules[hash(newRule->findrep)];
       }
+
+      while (*offsetAddress) {
+        ContractionTableRule *currentRule = CTR(tableHeader, *offsetAddress);
+        if (newRule->findlen > currentRule->findlen) break;
+        if (newRule->findlen == currentRule->findlen) {
+          if ((currentRule->opcode == CTO_Always) && (newRule->opcode != CTO_Always)) break;
+        }
+        offsetAddress = &currentRule->next;
+      }
+
+      newRule->next = *offsetAddress;
+      *offsetAddress = ruleOffset;
     }
 
-    return 1;
+    return newRule;
   }
-  return 0;
+  return NULL;
+}
+
+static const struct CharacterClass *
+findCharacterClass (const unsigned char *name, int length) {
+  const struct CharacterClass *class = characterClasses;
+  while (class) {
+    if ((length == class->length) &&
+        (memcmp(name, class->name, length) == 0))
+      return class;
+    class = class->next;
+  }
+  return NULL;
+}
+
+static struct CharacterClass *
+addCharacterClass (FileData *data, const unsigned char *name, int length) {
+  struct CharacterClass *class;
+  if (characterClassAttribute) {
+    if ((class = malloc(sizeof(*class) + length - 1))) {
+      memset(class, 0, sizeof(*class));
+      memcpy(class->name, name, (class->length = length));
+
+      class->attribute = characterClassAttribute;
+      characterClassAttribute <<= 1;
+
+      class->next = characterClasses;
+      characterClasses = class;
+      return class;
+    }
+  }
+  compileError(data, "character class table overflow: %.*s", length, name);
+  return NULL;
+}
+
+static void
+deallocateCharacterClasses (void) {
+  while (characterClasses) {
+    struct CharacterClass *class = characterClasses;
+    characterClasses = characterClasses->next;
+    free(class);
+  }
+}
+
+static int
+allocateCharacterClasses (void) {
+  const char *const *name = characterClassNames;
+  characterClasses = NULL;
+  characterClassAttribute = 1;
+  while (*name) {
+    if (!addCharacterClass(NULL, (const unsigned char *)*name, strlen(*name))) {
+      deallocateCharacterClasses();
+      return 0;
+    }
+    ++name;
+  }
+  return 1;
 }
 
 static ContractionTableOpcode
-getOpcode (FileData *data, const char *token, int length) {
-  BYTE *entry = opcodesTable;		/*pointer for looking things up in opcodesTable */
-  if (entry) {
-    const BYTE *end = entry + opcodesUsed;
-    while (entry < end) {
-      BYTE len = *entry++;
-      if (len == length)
-        if (memcmp(entry, token, length) == 0)
-          return *(entry + len);
-      entry += len + 1;
-    }
-  }
+getOpcode (FileData *data, const unsigned char *token, int length) {
+  ContractionTableOpcode opcode;
+  for (opcode=0; opcode<CTO_None; ++opcode)
+    if (length == opcodeLengths[opcode])
+      if (memcmp(token, opcodeNames[opcode], length) == 0)
+        return opcode;
   compileError(data, "opcode not defined: %.*s", length, token);
   return CTO_None;
 }
 
 static int
-addOpcode (FileData *data, const char *token, int length, ContractionTableOpcode opcode) {
-  int size = opcodesUsed + length + 2;
-  if (size > opcodesSize) {
-    void *table = realloc(opcodesTable, size|=0XFF);
-    if (!table) {
-      compileError(data, "Not enough memory for opcodes table.");
-      return 0;
-    }
-    opcodesTable = table;
-    opcodesSize = size;
-  }
-  opcodesTable[opcodesUsed++] = length;
-  memcpy(&opcodesTable[opcodesUsed], token, length);
-  opcodesUsed += length;
-  opcodesTable[opcodesUsed++] = opcode;
-  return 1;
-}
-
-static int
-getToken (FileData *data, const char **token, int *length, const char *description) { /*find the next string of contiguous nonblank characters */
-  const char *start = *token + *length;
+getToken (FileData *data, const unsigned char **token, int *length, const char *description) { /*find the next string of contiguous nonblank characters */
+  const unsigned char *start = *token + *length;
   int count = 0;
 
   while (isspace(*start)) start++;
@@ -261,7 +331,7 @@ octalDigit (FileData *data, int *value, char digit) {
 }
 
 static int
-parseText (FileData *data, ByteString *result, const char *token, const int length) {	/*interpret find string */
+parseText (FileData *data, ByteString *result, const unsigned char *token, const int length) {	/*interpret find string */
   int count = 0;		/*loop counters */
   int index;		/*loop counters */
   for (index = 0; index < length; index++) {
@@ -337,7 +407,7 @@ parseText (FileData *data, ByteString *result, const char *token, const int leng
 }				/*find string interpreted */
 
 static int
-parseDots (FileData *data, ByteString *cells, const char *token, const int length) {	/*get dot patterns */
+parseDots (FileData *data, ByteString *cells, const unsigned char *token, const int length) {	/*get dot patterns */
   BYTE cell = 0;		/*assembly place for dots */
   int count = 0;		/*loop counters */
   int index;		/*loop counters */
@@ -350,28 +420,28 @@ parseDots (FileData *data, ByteString *cells, const char *token, const int lengt
       {
         int dot;
       case '1':
-        dot = B1;
+        dot = BRL_DOT1;
         goto haveDot;
       case '2':
-        dot = B2;
+        dot = BRL_DOT2;
         goto haveDot;
       case '3':
-        dot = B3;
+        dot = BRL_DOT3;
         goto haveDot;
       case '4':
-        dot = B4;
+        dot = BRL_DOT4;
         goto haveDot;
       case '5':
-        dot = B5;
+        dot = BRL_DOT5;
         goto haveDot;
       case '6':
-        dot = B6;
+        dot = BRL_DOT6;
         goto haveDot;
       case '7':
-        dot = B7;
+        dot = BRL_DOT7;
         goto haveDot;
       case '8':
-        dot = B8;
+        dot = BRL_DOT8;
       haveDot:
         if (started && !cell) goto invalid;
         if (cell & dot) {
@@ -410,7 +480,15 @@ parseDots (FileData *data, ByteString *cells, const char *token, const int lengt
 }				/*end of function parseDots */
 
 static int
-getFindText (FileData *data, ByteString *find, const char **token, int *length) {
+getCharacters (FileData *data, ByteString *characters, const unsigned char **token, int *length) {
+  if (getToken(data, token, length, "characters"))
+    if (parseText(data, characters, *token, *length))
+      return 1;
+  return 0;
+}
+
+static int
+getFindText (FileData *data, ByteString *find, const unsigned char **token, int *length) {
   if (getToken(data, token, length, "find text"))
     if (parseText(data, find, *token, *length))
       return 1;
@@ -418,7 +496,7 @@ getFindText (FileData *data, ByteString *find, const char **token, int *length)
 }
 
 static int
-getReplaceText (FileData *data, ByteString *replace, const char **token, int *length) {
+getReplaceText (FileData *data, ByteString *replace, const unsigned char **token, int *length) {
   if (getToken(data, token, length, "replacement text"))
     if (parseText(data, replace, *token, *length))
       return 1;
@@ -426,7 +504,7 @@ getReplaceText (FileData *data, ByteString *replace, const char **token, int *le
 }
 
 static int
-getReplacePattern (FileData *data, ByteString *replace, const char **token, int *length) {
+getReplacePattern (FileData *data, ByteString *replace, const unsigned char **token, int *length) {
   if (getToken(data, token, length, "replacement pattern")) {
     if (*length == 1 && **token == '=') {
       replace->length = 0;
@@ -438,43 +516,20 @@ getReplacePattern (FileData *data, ByteString *replace, const char **token, int
   return 0;
 }
 
-static int
-integerToken (FileData *data, int *integer, const char *token, int length,
-              const char *description, const int *minimum, const int *maximum) {
-  char *end;
-  unsigned long value = strtol(token, &end, 0);
-  if (!minimum) {
-    static const int limit = 0;
-    minimum = &limit;
-  }
-  if (!maximum) {
-    static const int limit = 0X7FFFFFFF;
-    maximum = &limit;
-  }
-  if ((end != (token + length)) || (value < *minimum) || (value > *maximum)) {
-    compileError(data, "invalid %s: %.*s", description, length, token);
-    return 0;
-  }
-  *integer = value;
-  return 1;
-}
-
-static int
-opcodeToken (FileData *data, ContractionTableOpcode *opcode, const char *token, int length) {
-  static const int minimum = 0;
-  static const int maximum = CTO_None - 1;
-  int integer;
-  if (!integerToken(data, &integer, token, length, "opcode number", &minimum, &maximum))
-    return 0;
-  *opcode = integer;
-  return 1;
-}
-
 static const char *
 setLocale (const char *locale) {
   return setlocale(LC_CTYPE, locale);
 }
 
+static int
+getCharacterClass (FileData *data, const struct CharacterClass **class, const unsigned char **token, int *length) {
+  if (getToken(data, token, length, "character class name")) {
+    if ((*class = findCharacterClass(*token, *length))) return 1;
+    compileError(data, "character class not defined: %.*s", *length, *token);
+  }
+  return 0;
+}
+
 static const char *
 getLocale (void) {
   return setLocale(NULL);
@@ -485,7 +540,7 @@ static int
 includeFile (FileData *data, ByteString *path) {
   const char *prefixAddress = data->fileName;
   int prefixLength = 0;
-  const char *suffixAddress = path->bytes;
+  const unsigned char *suffixAddress = path->bytes;
   int suffixLength = path->length;
   if (*suffixAddress != '/') {
     const char *ptr = strrchr(prefixAddress, '/');
@@ -502,20 +557,21 @@ includeFile (FileData *data, ByteString *path) {
 static int
 processLine (FileData *data, const char *line) {
   int ok = 0;
-  const char *token = line;
+  const unsigned char *token = (const unsigned char *)line;
   int length = 0;			/*length of token */
   ContractionTableOpcode opcode;
+  ContractionTableCharacterAttributes after = 0;
+  ContractionTableCharacterAttributes before = 0;
 
+doOpcode:
   if (!getToken(data, &token, &length, NULL)) return 1;			/*blank line */
   if (*token == '#') return 1;
+  opcode = getOpcode(data, token, length);
 
-  if (*token < '0' || *token > '9') { /*look up word in opcode table */
-    opcode = getOpcode(data, token, length);
-  } else if (!opcodeToken(data, &opcode, token, length))
-    return 1;
   switch (opcode) { /*Carry out operations */
     case CTO_None:
       break;
+
     case CTO_IncludeFile: {
       ByteString path;
       if (getToken(data, &token, &length, "include file path"))
@@ -524,6 +580,7 @@ processLine (FileData *data, const char *line) {
             goto failure;
       break;
     }
+
     case CTO_Locale: {
       ByteString locale;
       if (getToken(data, &token, &length, "locale specification"))
@@ -539,55 +596,53 @@ processLine (FileData *data, const char *line) {
         }
       break;
     }
-    case CTO_Synonym:
-      if (getToken(data, &token, &length, "opcode number"))
-        if (opcodeToken(data, &opcode, token, length))
-          if (getToken(data, &token, &length, "opcode name"))
-            if (opcode != CTO_None)
-              if (!addOpcode(data, token, length, opcode))
-                goto failure;
-      break;
+
     case CTO_Always:
-    case CTO_WholeWord:
-    case CTO_LowWord:
-    case CTO_JoinableWord:
     case CTO_LargeSign:
+    case CTO_WholeWord:
+    case CTO_JoinableWord:
+    case CTO_LowWord:
     case CTO_SuffixableWord:
+    case CTO_PrefixableWord:
     case CTO_BegWord:
+    case CTO_BegMidWord:
     case CTO_MidWord:
     case CTO_MidEndWord:
     case CTO_EndWord:
+    case CTO_PrePunc:
+    case CTO_PostPunc:
     case CTO_BegNum:
     case CTO_MidNum:
     case CTO_EndNum:
-    case CTO_PrePunc:
-    case CTO_PostPunc:
     case CTO_Repeated: {
       ByteString find;
       ByteString replace;
       if (getFindText(data, &find, &token, &length))
         if (getReplacePattern(data, &replace, &token, &length))
-          if (!addRule(data, opcode, &find, &replace))
+          if (!addRule(data, opcode, &find, &replace, after, before))
             goto failure;
       break;
     }
+
     case CTO_Replace: {
       ByteString find;
       ByteString replace;
       if (getFindText(data, &find, &token, &length))
         if (getReplaceText(data, &replace, &token, &length))
-          if (!addRule(data, opcode, &find, &replace))
+          if (!addRule(data, opcode, &find, &replace, after, before))
             goto failure;
       break;
     }
+
     case CTO_Contraction:
     case CTO_Literal: {
       ByteString find;
       if (getFindText(data, &find, &token, &length))
-        if (!addRule(data, opcode, &find, NULL))
+        if (!addRule(data, opcode, &find, NULL, after, before))
           goto failure;
       break;
     }
+
     case CTO_CapitalSign: {
       ByteString cells;
       if (getToken(data, &token, &length, "capital sign"))
@@ -596,6 +651,7 @@ processLine (FileData *data, const char *line) {
             goto failure;
       break;
     }
+
     case CTO_BeginCapitalSign: {
       ByteString cells;
       if (getToken(data, &token, &length, "begin capital sign"))
@@ -604,6 +660,7 @@ processLine (FileData *data, const char *line) {
             goto failure;
       break;
     }
+
     case CTO_EndCapitalSign: {
       ByteString cells;
       if (getToken(data, &token, &length, "end capital sign"))
@@ -612,6 +669,7 @@ processLine (FileData *data, const char *line) {
             goto failure;
       break;
     }
+
     case CTO_EnglishLetterSign: {
       ByteString cells;
       if (getToken(data, &token, &length, "letter sign"))
@@ -620,6 +678,7 @@ processLine (FileData *data, const char *line) {
             goto failure;
       break;
     }
+
     case CTO_NumberSign: {
       ByteString cells;
       if (getToken(data, &token, &length, "number sign"))
@@ -628,8 +687,46 @@ processLine (FileData *data, const char *line) {
             goto failure;
       break;
     }
+
+    case CTO_Class: {
+      const struct CharacterClass *class;
+      ByteString characters;
+      if (getToken(data, &token, &length, "character class name")) {
+        if ((class = findCharacterClass(token, length))) {
+          compileError(data, "character class already defined: %.*s", length, token);
+        } else if ((class = addCharacterClass(data, token, length))) {
+          if (getCharacters(data, &characters, &token, &length)) {
+            int index;
+            for (index=0; index<characters.length; ++index) {
+              ContractionTableCharacter *character = &tableHeader->characters[characters.bytes[index]];
+              character->attributes |= class->attribute;
+            }
+          }
+        }
+      }
+      break;
+    }
+
+    {
+      ContractionTableCharacterAttributes *attributes;
+      const struct CharacterClass *class;
+
+    case CTO_After:
+      attributes = &after;
+      goto doClass;
+    case CTO_Before:
+      attributes = &before;
+    doClass:
+
+      if (getCharacterClass(data, &class, &token, &length)) {
+        *attributes |= class->attribute;
+        goto doOpcode;
+      }
+      break;
+    }
+
     default:
-      compileError(data, "unimplemented opcode: %d", opcode);
+      compileError(data, "unimplemented opcode: %.*s", length, token);
       break;
   }				/*end of loop for processing tableStream */
   ok = 1;
@@ -787,9 +884,11 @@ compileContractionTable (const char *fileName) { /*compile source table into a t
   tableSize = 0;
   tableUsed = 0;
 
-  opcodesTable = NULL;
-  opcodesSize = 0;
-  opcodesUsed = 0;
+  if (!opcodeLengths[0]) {
+    ContractionTableOpcode opcode;
+    for (opcode=0; opcode<CTO_None; ++opcode)
+      opcodeLengths[opcode] = strlen(opcodeNames[opcode]);
+  }
 
   originalLocale = getLocale();
   setLocale("C");
@@ -797,16 +896,19 @@ compileContractionTable (const char *fileName) { /*compile source table into a t
 
   if (allocateBytes(NULL, &headerOffset, sizeof(*tableHeader), __alignof__(*tableHeader))) {
     if (headerOffset == 0) {
-      if (processFile(fileName)) {
-        if (auditTable()) {
-          ok = 1;
-          if (!noLocale) {
-            if (saveString(NULL, &tableHeader->locale, getLocale()))
-              cacheCharacterAttributes();
-            else
-              ok = 0;
+      if (allocateCharacterClasses()) {
+        if (processFile(fileName)) {
+          if (auditTable()) {
+            ok = 1;
+            if (!noLocale) {
+              if (saveString(NULL, &tableHeader->locale, getLocale()))
+                cacheCharacterAttributes();
+              else
+                ok = 0;
+            }
           }
         }
+        deallocateCharacterClasses();
       }
     } else {
       compileError(NULL, "contraction table header not allocated at offset 0.");
@@ -815,18 +917,13 @@ compileContractionTable (const char *fileName) { /*compile source table into a t
 
   setLocale(originalLocale);
 
-  if (opcodesTable) {
-    free(opcodesTable);
-    opcodesTable = NULL;
-  }
-
   if (!ok) {
     free(tableHeader);
     tableHeader = NULL;
   }
 
   if (errorCount)
-    LogPrint(LOG_ERR, "%d %s in contraction table '%s'.",
+    LogPrint(LOG_WARNING, "%d %s in contraction table '%s'.",
              errorCount, ((errorCount == 1)? "error": "errors"), fileName);
   return (void *)tableHeader;
 }
