@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the Linux console (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2003 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -18,10 +18,16 @@
 #ifndef _HELP_H
 #define _HELP_H
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif /* HAVE_CONFIG_H */
+
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
 
+#include <inttypes.h>
+
 /* help.h - describes the helpfile format
  * $Id: help.h,v 1.3 1996/09/24 01:04:26 nn201 Exp $
  */
@@ -36,12 +42,20 @@ extern "C" {
 typedef struct {
   unsigned char pages;
   unsigned char unused;
-} __attribute__((packed)) HelpFileHeader;
+}
+#ifdef HAVE_ATTRIBUTE_PACKED
+  __attribute__((packed))
+#endif /* HAVE_ATTRIBUTE_PACKED */
+  HelpFileHeader;
 
 typedef struct {
-  unsigned char rows;
-  unsigned char columns;
-} __attribute__((packed)) HelpPageEntry;
+  uint16_t height;
+  uint16_t width;
+}
+#ifdef HAVE_ATTRIBUTE_PACKED
+  __attribute__((packed))
+#endif /* HAVE_ATTRIBUTE_PACKED */
+  HelpPageEntry;
 
 #ifdef __cplusplus
 }
