@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the Linux console (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2003 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -22,10 +22,16 @@
 extern "C" {
 #endif /* __cplusplus */
 
-extern volatile int csr_active;
-extern volatile pid_t csr_pid;
+extern int startCursorRouting (int column, int row, int screen);
+extern volatile pid_t routingProcess;
+extern volatile int routingStatus;
 
-int csrjmp (int x, int y, int scrno);
+typedef enum {
+  ROUTE_OK,
+  ROUTE_WRONG_COLUMN,
+  ROUTE_WRONG_ROW,
+  ROUTE_ERROR
+} RoutingStatus;
 
 #ifdef __cplusplus
 }
