@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the Linux console (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2003 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -34,6 +34,7 @@
 #include "scr.h"
 #include "scr_real.h"
 #include "scr_linux.h"
+#include "brldefs.h"
 
 static unsigned int debugCharacterTranslationTable = 0;
 static unsigned int debugApplicationCharacterMap = 0;
@@ -378,7 +379,7 @@ getUserAcm (int force) {
   if (controlConsole(GIO_UNISCRNMAP, &map) != -1) {
     if (force || (memcmp(applicationCharacterMap, map, sizeof(applicationCharacterMap)) != 0)) {
       memcpy(applicationCharacterMap, map, sizeof(applicationCharacterMap));
-      LogPrint(LOG_DEBUG, "Application character map changed.");
+      if (!force) LogPrint(LOG_DEBUG, "User application character map changed.");
       logApplicationCharacterMap();
       return 1;
     }
@@ -412,15 +413,14 @@ determineApplicationCharacterMap (int force) {
   return 1;
 }
 
-static int fontTableSize;
-static int isBigFontTable;
+static int vgaCharacterCount;
+static int vgaLargeTable;
 static int
-setFontTableSize (void) {
-  int ok = 0;
+setVgaCharacterCount (int force) {
   struct console_font_op cfo;
-
-  fontTableSize = 0X100;
-  isBigFontTable = 0;
+  int oldCount = vgaCharacterCount;
+  vgaCharacterCount = 0X100;
+  vgaLargeTable = 0;
 
   memset(&cfo, 0, sizeof(cfo));
   cfo.op = KD_FONT_OP_GET;
@@ -429,26 +429,26 @@ setFontTableSize (void) {
 
   if (controlConsole(KDFONTOP, &cfo) != -1) {
     switch (cfo.charcount) {
-      case 0X200:
-        isBigFontTable = 1;
-      case 0X100:
-        fontTableSize = cfo.charcount;
-      case 0X000:
-        ok = 1;
-        break;
       default:
-        LogPrint(LOG_WARNING, "Unexpected font table size: %d", cfo.charcount);
+        LogPrint(LOG_WARNING, "Unexpected VGA character count: %d", cfo.charcount);
+      case 0X200:
+        vgaLargeTable = 1;
+      case 0X100:
+        vgaCharacterCount = cfo.charcount;
+      case 0X000:
         break;
     }
-  } else {
-    LogPrint((errno == EINVAL)? LOG_DEBUG: LOG_WARNING,
-             "ioctl KDFONTOP[GET]: %s", strerror(errno));
+  } else if (errno != EINVAL) {
+    LogPrint(LOG_WARNING, "ioctl KDFONTOP[GET]: %s", strerror(errno));
   }
 
-  LogPrint(LOG_INFO, "Font Table Size: %d(%s)",
-           fontTableSize,
-           isBigFontTable? "big": "small");
-  return ok;
+  if (!force)
+    if (vgaCharacterCount == oldCount)
+      return 0;
+  LogPrint(LOG_INFO, "VGA Character Count: %d(%s)",
+           vgaCharacterCount,
+           vgaLargeTable? "large": "small");
+  return 1;
 }
 
 static struct unipair *screenFontMapTable;
@@ -581,14 +581,11 @@ static int
 setTranslationTable (int force) {
   int acmChanged = setApplicationCharacterMap && setApplicationCharacterMap(force);
   int sfmChanged = setScreenFontMap(force);
+  int vccChanged = setVgaCharacterCount(force);
 
-  if (sfmChanged) {
-    setFontTableSize();
-  }
-
-  if (acmChanged || sfmChanged) {
+  if (acmChanged || sfmChanged || vccChanged) {
     unsigned short directPosition = 0XFF;
-    if (isBigFontTable) directPosition |= 0X100;
+    if (vgaLargeTable) directPosition |= 0X100;
 
     memset(translationTable, '?', sizeof(translationTable));
     {
@@ -611,7 +608,7 @@ setTranslationTable (int force) {
              else if (map->unicode > unicode)
                last = current - 1;
              else {
-               if (map->fontpos < fontTableSize) position = map->fontpos;
+               if (map->fontpos < vgaCharacterCount) position = map->fontpos;
                break;
              }
            }
@@ -632,7 +629,7 @@ setTranslationTable (int force) {
     if (debugCharacterTranslationTable) {
       const unsigned int count = 0X10;
       int position;
-      for (position=0; position<fontTableSize; position+=count) {
+      for (position=0; position<vgaCharacterCount; position+=count) {
         char description[0X20];
         sprintf(description, "c2f[%02X]", position);
         LogBytes(description, &translationTable[position], count);
@@ -754,7 +751,7 @@ read_LinuxScreen (ScreenBox box, unsigned char *buffer, ScreenMode mode) {
           int column;
           for (column=0; column<box.width; ++column) {
             int position = *source;
-            if (isBigFontTable)
+            if (vgaLargeTable)
               if (source[1] & 0X08)
                 position |= 0X100;
             src[column] = *source;
@@ -773,7 +770,7 @@ read_LinuxScreen (ScreenBox box, unsigned char *buffer, ScreenMode mode) {
           int column;
           source++;
           for (column=0; column<box.width; ++column) {
-            if (isBigFontTable) *source &= 0XF7;
+            if (vgaLargeTable) *source &= 0XF7;
             *target++ = *source;
             source += 2;
           }
@@ -802,104 +799,123 @@ insertByte (unsigned char byte) {
 }
 
 static int
-insertCode (unsigned short key, int raw) {
+insertCode (ScreenKey key, int raw) {
   unsigned char prefix = 0X00;
   unsigned char code;
+  int modShift = 0;
+  int modControl = 0;
+  int modMeta = 0;
+
+  if (key < SCR_KEY_ENTER) {
+    if (key & SCR_KEY_MOD_META) {
+      key &= ~SCR_KEY_MOD_META;
+      modMeta = 1;
+    }
+
+    if ((key >= 'A') && (key <= 'Z')) {
+      key = (key - 'A') + 'a';
+      modShift = 1;
+    } else if (!(key & 0XE0)) {
+      key |= 0X60;
+      modControl = 1;
+    }
+  }
+
   switch (key) {
-    case KEY_ESCAPE:        code = 0X01; break;
-    case KEY_FUNCTION +  0: code = 0X3B; break;
-    case KEY_FUNCTION +  1: code = 0X3C; break;
-    case KEY_FUNCTION +  2: code = 0X3D; break;
-    case KEY_FUNCTION +  3: code = 0X3E; break;
-    case KEY_FUNCTION +  4: code = 0X3F; break;
-    case KEY_FUNCTION +  5: code = 0X40; break;
-    case KEY_FUNCTION +  6: code = 0X41; break;
-    case KEY_FUNCTION +  7: code = 0X42; break;
-    case KEY_FUNCTION +  8: code = 0X43; break;
-    case KEY_FUNCTION +  9: code = 0X44; break;
-    case KEY_FUNCTION + 10: code = 0X57; break;
-    case KEY_FUNCTION + 11: code = 0X58; break;
-    case '`':               code = 0X29; break;
-    case '1':               code = 0X02; break;
-    case '2':               code = 0X03; break;
-    case '3':               code = 0X04; break;
-    case '4':               code = 0X05; break;
-    case '5':               code = 0X06; break;
-    case '6':               code = 0X07; break;
-    case '7':               code = 0X08; break;
-    case '8':               code = 0X09; break;
-    case '9':               code = 0X0A; break;
-    case '0':               code = 0X0B; break;
-    case '-':               code = 0X0C; break;
-    case '=':               code = 0X0D; break;
-    case KEY_BACKSPACE:     code = 0X0E; break;
-    case KEY_TAB:           code = 0X0F; break;
-    case 'q':               code = 0X10; break;
-    case 'w':               code = 0X11; break;
-    case 'e':               code = 0X12; break;
-    case 'r':               code = 0X13; break;
-    case 't':               code = 0X14; break;
-    case 'y':               code = 0X15; break;
-    case 'u':               code = 0X16; break;
-    case 'i':               code = 0X17; break;
-    case 'o':               code = 0X18; break;
-    case 'p':               code = 0X19; break;
-    case '[':               code = 0X1A; break;
-    case ']':               code = 0X1B; break;
-    case '\\':              code = 0X2B; break;
-    case 'a':               code = 0X1E; break;
-    case 's':               code = 0X1F; break;
-    case 'd':               code = 0X20; break;
-    case 'f':               code = 0X21; break;
-    case 'g':               code = 0X22; break;
-    case 'h':               code = 0X23; break;
-    case 'j':               code = 0X24; break;
-    case 'k':               code = 0X25; break;
-    case 'l':               code = 0X26; break;
-    case ';':               code = 0X27; break;
-    case '\'':              code = 0X28; break;
-    case KEY_RETURN:        code = 0X1C; break;
-    case 'z':               code = 0X2C; break;
-    case 'x':               code = 0X2D; break;
-    case 'c':               code = 0X2E; break;
-    case 'v':               code = 0X2F; break;
-    case 'b':               code = 0X30; break;
-    case 'n':               code = 0X31; break;
-    case 'm':               code = 0X32; break;
-    case ',':               code = 0X33; break;
-    case '.':               code = 0X34; break;
-    case '/':               code = 0X35; break;
-    case ' ':               code = 0X39; break;
+    case SCR_KEY_ESCAPE:        code = 0X01; break;
+    case SCR_KEY_FUNCTION +  0: code = 0X3B; break;
+    case SCR_KEY_FUNCTION +  1: code = 0X3C; break;
+    case SCR_KEY_FUNCTION +  2: code = 0X3D; break;
+    case SCR_KEY_FUNCTION +  3: code = 0X3E; break;
+    case SCR_KEY_FUNCTION +  4: code = 0X3F; break;
+    case SCR_KEY_FUNCTION +  5: code = 0X40; break;
+    case SCR_KEY_FUNCTION +  6: code = 0X41; break;
+    case SCR_KEY_FUNCTION +  7: code = 0X42; break;
+    case SCR_KEY_FUNCTION +  8: code = 0X43; break;
+    case SCR_KEY_FUNCTION +  9: code = 0X44; break;
+    case SCR_KEY_FUNCTION + 10: code = 0X57; break;
+    case SCR_KEY_FUNCTION + 11: code = 0X58; break;
+    case '`':                   code = 0X29; break;
+    case '1':                   code = 0X02; break;
+    case '2':                   code = 0X03; break;
+    case '3':                   code = 0X04; break;
+    case '4':                   code = 0X05; break;
+    case '5':                   code = 0X06; break;
+    case '6':                   code = 0X07; break;
+    case '7':                   code = 0X08; break;
+    case '8':                   code = 0X09; break;
+    case '9':                   code = 0X0A; break;
+    case '0':                   code = 0X0B; break;
+    case '-':                   code = 0X0C; break;
+    case '=':                   code = 0X0D; break;
+    case SCR_KEY_BACKSPACE:     code = 0X0E; break;
+    case SCR_KEY_TAB:           code = 0X0F; break;
+    case 'q':                   code = 0X10; break;
+    case 'w':                   code = 0X11; break;
+    case 'e':                   code = 0X12; break;
+    case 'r':                   code = 0X13; break;
+    case 't':                   code = 0X14; break;
+    case 'y':                   code = 0X15; break;
+    case 'u':                   code = 0X16; break;
+    case 'i':                   code = 0X17; break;
+    case 'o':                   code = 0X18; break;
+    case 'p':                   code = 0X19; break;
+    case '[':                   code = 0X1A; break;
+    case ']':                   code = 0X1B; break;
+    case '\\':                  code = 0X2B; break;
+    case 'a':                   code = 0X1E; break;
+    case 's':                   code = 0X1F; break;
+    case 'd':                   code = 0X20; break;
+    case 'f':                   code = 0X21; break;
+    case 'g':                   code = 0X22; break;
+    case 'h':                   code = 0X23; break;
+    case 'j':                   code = 0X24; break;
+    case 'k':                   code = 0X25; break;
+    case 'l':                   code = 0X26; break;
+    case ';':                   code = 0X27; break;
+    case '\'':                  code = 0X28; break;
+    case SCR_KEY_ENTER:        code = 0X1C; break;
+    case 'z':                   code = 0X2C; break;
+    case 'x':                   code = 0X2D; break;
+    case 'c':                   code = 0X2E; break;
+    case 'v':                   code = 0X2F; break;
+    case 'b':                   code = 0X30; break;
+    case 'n':                   code = 0X31; break;
+    case 'm':                   code = 0X32; break;
+    case ',':                   code = 0X33; break;
+    case '.':                   code = 0X34; break;
+    case '/':                   code = 0X35; break;
+    case ' ':                   code = 0X39; break;
     default:
       if (raw) {
         prefix = 0XE0;
         switch (key) {
-          case KEY_INSERT:       code = 0X52; break;
-          case KEY_HOME:         code = 0X47; break;
-          case KEY_PAGE_UP:      code = 0X49; break;
-          case KEY_DELETE:       code = 0X53; break;
-          case KEY_END:          code = 0X4F; break;
-          case KEY_PAGE_DOWN:    code = 0X51; break;
-          case KEY_CURSOR_UP:    code = 0X48; break;
-          case KEY_CURSOR_LEFT:  code = 0X4B; break;
-          case KEY_CURSOR_DOWN:  code = 0X50; break;
-          case KEY_CURSOR_RIGHT: code = 0X4D; break;
+          case SCR_KEY_INSERT:       code = 0X52; break;
+          case SCR_KEY_HOME:         code = 0X47; break;
+          case SCR_KEY_PAGE_UP:      code = 0X49; break;
+          case SCR_KEY_DELETE:       code = 0X53; break;
+          case SCR_KEY_END:          code = 0X4F; break;
+          case SCR_KEY_PAGE_DOWN:    code = 0X51; break;
+          case SCR_KEY_CURSOR_UP:    code = 0X48; break;
+          case SCR_KEY_CURSOR_LEFT:  code = 0X4B; break;
+          case SCR_KEY_CURSOR_DOWN:  code = 0X50; break;
+          case SCR_KEY_CURSOR_RIGHT: code = 0X4D; break;
           default:
             LogPrint(LOG_WARNING, "Key %4.4X not suported in scancode mode.", key);
             return 0;
         }
       } else {
         switch (key) {
-          case KEY_INSERT:       code = 0X6E; break;
-          case KEY_HOME:         code = 0X66; break;
-          case KEY_PAGE_UP:      code = 0X68; break;
-          case KEY_DELETE:       code = 0X6F; break;
-          case KEY_END:          code = 0X6B; break;
-          case KEY_PAGE_DOWN:    code = 0X6D; break;
-          case KEY_CURSOR_UP:    code = 0X67; break;
-          case KEY_CURSOR_LEFT:  code = 0X69; break;
-          case KEY_CURSOR_DOWN:  code = 0X6C; break;
-          case KEY_CURSOR_RIGHT: code = 0X6A; break;
+          case SCR_KEY_INSERT:       code = 0X6E; break;
+          case SCR_KEY_HOME:         code = 0X66; break;
+          case SCR_KEY_PAGE_UP:      code = 0X68; break;
+          case SCR_KEY_DELETE:       code = 0X6F; break;
+          case SCR_KEY_END:          code = 0X6B; break;
+          case SCR_KEY_PAGE_DOWN:    code = 0X6D; break;
+          case SCR_KEY_CURSOR_UP:    code = 0X67; break;
+          case SCR_KEY_CURSOR_LEFT:  code = 0X69; break;
+          case SCR_KEY_CURSOR_DOWN:  code = 0X6C; break;
+          case SCR_KEY_CURSOR_RIGHT: code = 0X6A; break;
           default:
             LogPrint(LOG_WARNING, "Key %4.4X not suported in keycode mode.", key);
             return 0;
@@ -907,14 +923,24 @@ insertCode (unsigned short key, int raw) {
       }
       break;
   }
+
   {
-    unsigned char buffer[4];
+    unsigned char buffer[10];
     unsigned short count = 0;
     const unsigned char *byte = buffer;
+
+    if (modControl) buffer[count++] = 0X1D;
+    if (modMeta) buffer[count++] = 0X38;
+    if (modShift) buffer[count++] = 0X2A;
     if (prefix) buffer[count++] = prefix;
     buffer[count++] = code;
+
     if (prefix) buffer[count++] = prefix;
     buffer[count++] = code | 0X80;
+    if (modShift) buffer[count++] = 0X2A | 0X80;
+    if (modMeta) buffer[count++] = 0X38 | 0X80;
+    if (modControl) buffer[count++] = 0X1D | 0X80;
+
     while (count--) {
       if (!insertByte(*byte++)) return 0;
     }
@@ -923,123 +949,147 @@ insertCode (unsigned short key, int raw) {
 }
 
 static int
-insertMapped (unsigned short key, int (*byteInserter)(unsigned char byte)) {
-  if (key < 0X100) {
-    unsigned char character = key & 0XFF;
-    if (!byteInserter(character)) return 0;
+insertMapped (ScreenKey key, int (*byteInserter)(unsigned char byte)) {
+  unsigned char buffer[2];
+  unsigned char *sequence;
+  unsigned char *end;
+
+  if (key < SCR_KEY_ENTER) {
+    sequence = end = buffer + sizeof(buffer);
+    *--sequence = key & 0XFF;
+
+    if (key & SCR_KEY_MOD_META) {
+      long meta;
+      if (controlConsole(KDGKBMETA, &meta) == -1) return 0;
+
+      switch (meta) {
+        case K_METABIT:
+          *sequence |= 0X80;
+          break;
+
+        case K_ESCPREFIX:
+          *--sequence = 0X1B;
+          break;
+
+        default:
+          LogPrint(LOG_WARNING, "Unsupported keyboard meta mode: %ld", meta);
+          return 0;
+      }
+    }
   } else {
-    const char *sequence;
     switch (key) {
-      case KEY_RETURN:
+      case SCR_KEY_ENTER:
         sequence = "\r";
         break;
-      case KEY_TAB:
+      case SCR_KEY_TAB:
         sequence = "\t";
         break;
-      case KEY_BACKSPACE:
-        sequence = "\177";
+      case SCR_KEY_BACKSPACE:
+        sequence = "\x7f";
         break;
-      case KEY_ESCAPE:
-        sequence = "\033";
+      case SCR_KEY_ESCAPE:
+        sequence = "\x1b";
         break;
-      case KEY_CURSOR_LEFT:
-        sequence = "\033[D";
+      case SCR_KEY_CURSOR_LEFT:
+        sequence = "\x1b[D";
         break;
-      case KEY_CURSOR_RIGHT:
-        sequence = "\033[C";
+      case SCR_KEY_CURSOR_RIGHT:
+        sequence = "\x1b[C";
         break;
-      case KEY_CURSOR_UP:
-        sequence = "\033[A";
+      case SCR_KEY_CURSOR_UP:
+        sequence = "\x1b[A";
         break;
-      case KEY_CURSOR_DOWN:
-        sequence = "\033[B";
+      case SCR_KEY_CURSOR_DOWN:
+        sequence = "\x1b[B";
         break;
-      case KEY_PAGE_UP:
-        sequence = "\033[5~";
+      case SCR_KEY_PAGE_UP:
+        sequence = "\x1b[5~";
         break;
-      case KEY_PAGE_DOWN:
-        sequence = "\033[6~";
+      case SCR_KEY_PAGE_DOWN:
+        sequence = "\x1b[6~";
         break;
-      case KEY_HOME:
-        sequence = "\033[1~";
+      case SCR_KEY_HOME:
+        sequence = "\x1b[1~";
         break;
-      case KEY_END:
-        sequence = "\033[4~";
+      case SCR_KEY_END:
+        sequence = "\x1b[4~";
         break;
-      case KEY_INSERT:
-        sequence = "\033[2~";
+      case SCR_KEY_INSERT:
+        sequence = "\x1b[2~";
         break;
-      case KEY_DELETE:
-        sequence = "\033[3~";
+      case SCR_KEY_DELETE:
+        sequence = "\x1b[3~";
         break;
-      case KEY_FUNCTION + 0:
-        sequence = "\033OP";
+      case SCR_KEY_FUNCTION + 0:
+        sequence = "\x1bOP";
         break;
-      case KEY_FUNCTION + 1:
-        sequence = "\033OQ";
+      case SCR_KEY_FUNCTION + 1:
+        sequence = "\x1bOQ";
         break;
-      case KEY_FUNCTION + 2:
-        sequence = "\033OR";
+      case SCR_KEY_FUNCTION + 2:
+        sequence = "\x1bOR";
         break;
-      case KEY_FUNCTION + 3:
-        sequence = "\033OS";
+      case SCR_KEY_FUNCTION + 3:
+        sequence = "\x1bOS";
         break;
-      case KEY_FUNCTION + 4:
-        sequence = "\033[15~";
+      case SCR_KEY_FUNCTION + 4:
+        sequence = "\x1b[15~";
         break;
-      case KEY_FUNCTION + 5:
-        sequence = "\033[17~";
+      case SCR_KEY_FUNCTION + 5:
+        sequence = "\x1b[17~";
         break;
-      case KEY_FUNCTION + 6:
-        sequence = "\033[18~";
+      case SCR_KEY_FUNCTION + 6:
+        sequence = "\x1b[18~";
         break;
-      case KEY_FUNCTION + 7:
-        sequence = "\033[19~";
+      case SCR_KEY_FUNCTION + 7:
+        sequence = "\x1b[19~";
         break;
-      case KEY_FUNCTION + 8:
-        sequence = "\033[20~";
+      case SCR_KEY_FUNCTION + 8:
+        sequence = "\x1b[20~";
         break;
-      case KEY_FUNCTION + 9:
-        sequence = "\033[21~";
+      case SCR_KEY_FUNCTION + 9:
+        sequence = "\x1b[21~";
         break;
-      case KEY_FUNCTION + 10:
-        sequence = "\033[23~";
+      case SCR_KEY_FUNCTION + 10:
+        sequence = "\x1b[23~";
         break;
-      case KEY_FUNCTION + 11:
-        sequence = "\033[24~";
+      case SCR_KEY_FUNCTION + 11:
+        sequence = "\x1b[24~";
         break;
-      case KEY_FUNCTION + 12:
-        sequence = "\033[25~";
+      case SCR_KEY_FUNCTION + 12:
+        sequence = "\x1b[25~";
         break;
-      case KEY_FUNCTION + 13:
-        sequence = "\033[26~";
+      case SCR_KEY_FUNCTION + 13:
+        sequence = "\x1b[26~";
         break;
-      case KEY_FUNCTION + 14:
-        sequence = "\033[28~";
+      case SCR_KEY_FUNCTION + 14:
+        sequence = "\x1b[28~";
         break;
-      case KEY_FUNCTION + 15:
-        sequence = "\033[29~";
+      case SCR_KEY_FUNCTION + 15:
+        sequence = "\x1b[29~";
         break;
-      case KEY_FUNCTION + 16:
-        sequence = "\033[31~";
+      case SCR_KEY_FUNCTION + 16:
+        sequence = "\x1b[31~";
         break;
-      case KEY_FUNCTION + 17:
-        sequence = "\033[32~";
+      case SCR_KEY_FUNCTION + 17:
+        sequence = "\x1b[32~";
         break;
-      case KEY_FUNCTION + 18:
-        sequence = "\033[33~";
+      case SCR_KEY_FUNCTION + 18:
+        sequence = "\x1b[33~";
         break;
-      case KEY_FUNCTION + 19:
-        sequence = "\033[34~";
+      case SCR_KEY_FUNCTION + 19:
+        sequence = "\x1b[34~";
         break;
       default:
         LogPrint(LOG_WARNING, "Key %4.4X not suported in ANSI mode.", key);
         return 0;
     }
-    while (*sequence) {
-      if (!byteInserter(*sequence++)) return 0;
-    }
+    end = sequence + strlen(sequence);
   }
+
+  while (sequence != end)
+    if (!byteInserter(*sequence++))
+      return 0;
   return 1;
 }
 
@@ -1054,7 +1104,7 @@ insertUtf8 (unsigned char byte) {
 }
 
 static int
-insert_LinuxScreen (unsigned short key) {
+insert_LinuxScreen (ScreenKey key) {
   int ok = 0;
   LogPrint(LOG_DEBUG, "Insert key: %4.4X", key);
   if (rebindConsole()) {
@@ -1120,6 +1170,216 @@ currentvt_LinuxScreen (void) {
   return description.no;
 }
 
+#ifdef HAVE_LINUX_INPUT_H
+#include <linux/input.h>
+
+typedef unsigned char At2KeyTable[0X100];
+
+static const At2KeyTable at2KeysOriginal = {
+  [0X76] = KEY_ESC,
+  [0X05] = KEY_F1,
+  [0X06] = KEY_F2,
+  [0X04] = KEY_F3,
+  [0X0C] = KEY_F4,
+  [0X03] = KEY_F5,
+  [0X0B] = KEY_F6,
+  [0X83] = KEY_F7,
+  [0X0A] = KEY_F8,
+  [0X01] = KEY_F9,
+  [0X09] = KEY_F10,
+  [0X78] = KEY_F11,
+  [0X07] = KEY_F12,
+  [0X7E] = KEY_SCROLLLOCK,
+
+  [0X0E] = KEY_GRAVE,
+  [0X16] = KEY_1,
+  [0X1E] = KEY_2,
+  [0X26] = KEY_3,
+  [0X25] = KEY_4,
+  [0X2E] = KEY_5,
+  [0X36] = KEY_6,
+  [0X3D] = KEY_7,
+  [0X3E] = KEY_8,
+  [0X46] = KEY_9,
+  [0X45] = KEY_0,
+  [0X4E] = KEY_MINUS,
+  [0X55] = KEY_EQUAL,
+  [0X66] = KEY_BACKSPACE,
+
+  [0X0D] = KEY_TAB,
+  [0X15] = KEY_Q,
+  [0X1D] = KEY_W,
+  [0X24] = KEY_E,
+  [0X2D] = KEY_R,
+  [0X2C] = KEY_T,
+  [0X35] = KEY_Y,
+  [0X3C] = KEY_U,
+  [0X43] = KEY_I,
+  [0X44] = KEY_O,
+  [0X4D] = KEY_P,
+  [0X54] = KEY_LEFTBRACE,
+  [0X5B] = KEY_RIGHTBRACE,
+  [0X5D] = KEY_BACKSLASH,
+
+  [0X58] = KEY_CAPSLOCK,
+  [0X1C] = KEY_A,
+  [0X1B] = KEY_S,
+  [0X23] = KEY_D,
+  [0X2B] = KEY_F,
+  [0X34] = KEY_G,
+  [0X33] = KEY_H,
+  [0X3B] = KEY_J,
+  [0X42] = KEY_K,
+  [0X4B] = KEY_L,
+  [0X4C] = KEY_SEMICOLON,
+  [0X52] = KEY_APOSTROPHE,
+  [0X5A] = KEY_ENTER,
+
+  [0X12] = KEY_LEFTSHIFT,
+  [0X61] = KEY_102ND,
+  [0X1A] = KEY_Z,
+  [0X22] = KEY_X,
+  [0X21] = KEY_C,
+  [0X2A] = KEY_V,
+  [0X32] = KEY_B,
+  [0X31] = KEY_N,
+  [0X3A] = KEY_M,
+  [0X41] = KEY_COMMA,
+  [0X49] = KEY_DOT,
+  [0X4A] = KEY_SLASH,
+  [0X59] = KEY_RIGHTSHIFT,
+
+  [0X14] = KEY_LEFTCTRL,
+  [0X11] = KEY_LEFTALT,
+  [0X29] = KEY_SPACE,
+
+  [0X77] = KEY_NUMLOCK,
+  [0X7C] = KEY_KPASTERISK,
+  [0X7B] = KEY_KPMINUS,
+  [0X79] = KEY_KPPLUS,
+  [0X71] = KEY_KPDOT,
+  [0X70] = KEY_KP0,
+  [0X69] = KEY_KP1,
+  [0X72] = KEY_KP2,
+  [0X7A] = KEY_KP3,
+  [0X6B] = KEY_KP4,
+  [0X73] = KEY_KP5,
+  [0X74] = KEY_KP6,
+  [0X6C] = KEY_KP7,
+  [0X75] = KEY_KP8,
+  [0X7D] = KEY_KP9
+};
+
+static const At2KeyTable at2KeysE0 = {
+  [0X1F] = KEY_LEFTMETA,
+  [0X11] = KEY_RIGHTALT,
+  [0X27] = KEY_RIGHTMETA,
+  [0X2F] = KEY_COMPOSE,
+  [0X14] = KEY_RIGHTCTRL,
+  [0X70] = KEY_INSERT,
+  [0X71] = KEY_DELETE,
+  [0X6C] = KEY_HOME,
+  [0X69] = KEY_END,
+  [0X7D] = KEY_PAGEUP,
+  [0X7A] = KEY_PAGEDOWN,
+  [0X75] = KEY_UP,
+  [0X6B] = KEY_LEFT,
+  [0X72] = KEY_DOWN,
+  [0X74] = KEY_RIGHT,
+  [0X4A] = KEY_KPSLASH,
+  [0X5A] = KEY_KPENTER
+};
+
+static const char *at2Keys;
+static int at2Pressed;
+#endif /* HAVE_LINUX_INPUT_H */
+
+#ifdef HAVE_LINUX_UINPUT_H
+#include <linux/uinput.h>
+
+static int uinputDevice = -1;
+
+static int
+openUinputDevice (void) {
+  if (uinputDevice != -1) return 1;
+
+  if ((uinputDevice = openDevice("/dev/uinput", "uinput", O_WRONLY, 10, 223)) != -1) {
+    struct uinput_user_dev device;
+    
+    memset(&device, 0, sizeof(device));
+    strcpy(device.name, "brltty");
+    if (write(uinputDevice, &device, sizeof(device)) == sizeof(device)) {
+      ioctl(uinputDevice, UI_SET_EVBIT, EV_KEY);
+      ioctl(uinputDevice, UI_SET_EVBIT, EV_REP);
+      {
+        int key;
+        for (key=KEY_RESERVED; key<KEY_UNKNOWN; key++) {
+          ioctl(uinputDevice, UI_SET_KEYBIT, key);
+        }
+      }
+
+      if (ioctl(uinputDevice, UI_DEV_CREATE) != -1) {
+        return 1;
+      } else {
+        LogError("ioctl UI_DEV_CREATE");
+      }
+    } else {
+      LogError("uinput_user_dev write");
+    }
+
+    close(uinputDevice);
+    uinputDevice = -1;
+  }
+  return 0;
+}
+#endif /* HAVE_LINUX_UINPUT_H */
+
+static int
+execute_LinuxScreen (int command) {
+  int blk = command & BRL_MSK_BLK;
+  int arg
+#ifdef HAVE_ATTRIBUTE_UNUSED
+      __attribute__((unused))
+#endif /* HAVE_ATTRIBUTE_UNUSED */
+      = command & BRL_MSK_ARG;
+
+  switch (blk) {
+    case BRL_BLK_PASSAT2:
+#ifdef HAVE_LINUX_INPUT_H
+      if (arg == 0XF0) {
+        at2Pressed = 0;
+      } else if (arg == 0XE0) {
+        at2Keys = at2KeysE0;
+      } else {
+        unsigned char key = at2Keys[arg];
+        int pressed
+#ifdef HAVE_ATTRIBUTE_UNUSED
+            __attribute__((unused))
+#endif /* HAVE_ATTRIBUTE_UNUSED */
+            = at2Pressed;
+
+        at2Keys = at2KeysOriginal;
+        at2Pressed = 1;
+
+        if (key) {
+#ifdef HAVE_LINUX_UINPUT_H
+          if (openUinputDevice()) {
+            struct input_event event;
+            event.type = EV_KEY;
+            event.code = key;
+            event.value = pressed;
+            write(uinputDevice, &event, sizeof(event));
+            return 1;
+          }
+#endif /* HAVE_LINUX_UINPUT_H */
+        }
+      }
+#endif /* HAVE_LINUX_INPUT_H */
+      break;
+  }
+  return 0;
+}
+
 void
 initializeLiveScreen (MainScreen *main) {
   initializeRealScreen(main);
@@ -1129,9 +1389,15 @@ initializeLiveScreen (MainScreen *main) {
   main->base.selectvt = selectvt_LinuxScreen;
   main->base.switchvt = switchvt_LinuxScreen;
   main->base.currentvt = currentvt_LinuxScreen;
+  main->base.execute = execute_LinuxScreen;
   main->parameters = parameters_LinuxScreen;
   main->prepare = prepare_LinuxScreen;
   main->open = open_LinuxScreen;
   main->setup = setup_LinuxScreen;
   main->close = close_LinuxScreen;
+
+#ifdef HAVE_LINUX_INPUT_H
+  at2Keys = at2KeysOriginal;
+  at2Pressed = 1;
+#endif /* HAVE_LINUX_INPUT_H */
 }
