@@ -3,7 +3,7 @@
 # BRLTTY - A background process providing access to the Linux console (when in
 #          text mode) for a blind person using a refreshable braille display.
 #
-# Copyright (C) 1995-2003 by The BRLTTY Team. All rights reserved.
+# Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
 #
 # BRLTTY comes with ABSOLUTELY NO WARRANTY.
 #
@@ -16,9 +16,11 @@
 # This software is maintained by Dave Mielke <dave@mielke.cc>.
 ###############################################################################
 
-cd "${0%/*}"
+set -e
+cd `dirname "${0}"`
+./gendeps
 [ -f Makefile ] && make -s distclean
 "${BRLTTY_AUTOCONF:-autoconf}"
-rm -fr autom4te*.cache
-./gendeps
+rm -fr autom4te*.cache || :
+[ "${#}" -gt 0 ] && ./configure "${@}"
 exit 0
